"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const kyso_model_1 = require("@kyso-io/kyso-model");
const kyso_store_1 = require("@kyso-io/kyso-store");
const core_1 = require("@oclif/core");
const jwt_decode_1 = tslib_1.__importDefault(require("jwt-decode"));
const inquirer = require("inquirer");
const helper_1 = require("../../helpers/helper");
const interactive_login_1 = require("../../helpers/interactive-login");
const kyso_command_1 = require("../kyso-command");
class AddChannel extends kyso_command_1.KysoCommand {
    async createChannel(api, organization, userId, channelDisplayName, yes) {
        const createTeam = new kyso_model_1.Team(channelDisplayName, '', '', '', '', [], organization.id, kyso_model_1.TeamVisibilityEnum.PROTECTED, userId, kyso_model_1.AllowDownload.ALL);
        // Don¡t show prompt
        if (!yes) {
            if (!createTeam.display_name) {
                const displayNameResponse = await inquirer.prompt([
                    {
                        type: 'input',
                        name: 'displayName',
                        message: 'What is the name of the channel?',
                        validate(displayName) {
                            if (displayName === '') {
                                return 'Name cannot be empty';
                            }
                            return true;
                        },
                    },
                ]);
                createTeam.display_name = displayNameResponse.displayName;
            }
            const teamVisibilityResponse = await inquirer.prompt([
                {
                    type: 'list',
                    name: 'teamVisibility',
                    message: `What is the visibility of the channel '${createTeam.display_name}'?`,
                    choices: [
                        {
                            name: 'Private: Only invited members of this channel have access to this channels content.',
                            value: kyso_model_1.TeamVisibilityEnum.PRIVATE,
                        },
                        {
                            name: `Organization only: All members of the organization ${organization.display_name} can access this channel`,
                            value: kyso_model_1.TeamVisibilityEnum.PROTECTED,
                        },
                        {
                            name: 'Public: Everyone can see this channel. Reports in this channel can be viewed by anyone with the reports url.',
                            value: kyso_model_1.TeamVisibilityEnum.PUBLIC,
                        },
                    ],
                },
            ]);
            createTeam.visibility = teamVisibilityResponse.teamVisibility;
            const bioResponse = await inquirer.prompt([
                {
                    type: 'input',
                    name: 'bio',
                    message: `What is the description of the channel '${createTeam.display_name}'?`,
                },
            ]);
            createTeam.bio = bioResponse.bio;
        }
        try {
            api.setOrganizationSlug(organization.sluglified_name);
            const result = await api.createTeam(createTeam);
            const team = result.data;
            const kysoCredentials = kyso_command_1.KysoCommand.getCredentials();
            this.log(`\nChannel '${team.display_name}' created. Visit its page ${kysoCredentials.kysoInstallUrl}/${organization.sluglified_name}/${team.sluglified_name}\n`);
        }
        catch (e) {
            this.log(`Error creating the channel '${createTeam.display_name}': ${e.response.data.message}`);
        }
    }
    async run() {
        const { args, flags } = await this.parse(AddChannel);
        await (0, interactive_login_1.launchInteractiveLoginIfNotLogged)();
        const kysoCredentials = kyso_command_1.KysoCommand.getCredentials();
        const result = await helper_1.Helper.getOrganizationFromSlugSecurely(args.organization, kysoCredentials);
        const slugifiedOrganization = result.data.sluglified_name;
        const channelsNames = args.list_of_channels.split(',');
        const api = new kyso_store_1.Api();
        api.configure(`${kysoCredentials.kysoInstallUrl}/api/v1`, kysoCredentials === null || kysoCredentials === void 0 ? void 0 : kysoCredentials.token);
        const decoded = (0, jwt_decode_1.default)(kysoCredentials.token);
        let tokenPermissions = null;
        try {
            const resultPermissions = await api.getUserPermissions(decoded.payload.username);
            tokenPermissions = resultPermissions.data;
        }
        catch (e) {
            this.error('Error getting user permissions');
        }
        const indexOrganization = tokenPermissions.organizations.findIndex((resourcePermissionOrganization) => resourcePermissionOrganization.name === slugifiedOrganization);
        if (indexOrganization === -1 && !helper_1.Helper.isGlobalAdmin(tokenPermissions)) {
            this.error(`You don't have permissions to create channels the organization ${args.organization}`);
        }
        const organizationResourcePermissions = tokenPermissions.organizations[indexOrganization];
        const indexPermissionCreateChannel = organizationResourcePermissions.permissions.findIndex((permission) => permission === kyso_model_1.TeamPermissionsEnum.CREATE);
        if (indexPermissionCreateChannel === -1) {
            this.error(`You don't have permissions to create channels for the organization ${args.organization}`);
        }
        let organization = null;
        try {
            const resultOrganization = await api.getOrganizationBySlug(slugifiedOrganization);
            organization = resultOrganization.data;
        }
        catch (e) {
            this.error(`Error getting the organization ${args.organization}`);
        }
        for (const channelDisplayName of channelsNames) {
            try {
                // Exists?
                const existsChannel = await helper_1.Helper.getChannelFromSlugSecurely(organization, channelDisplayName, kysoCredentials, true);
                this.log(`Channel ${existsChannel.data.display_name} already exists`);
            }
            catch (e) {
                // Don't exists... then let's create it
                try {
                    await this.createChannel(api, organization, decoded.payload.id, channelDisplayName, flags.yes);
                }
                catch (e) {
                    this.log(`Error creating channel ${channelDisplayName}`);
                }
            }
        }
    }
}
exports.default = AddChannel;
AddChannel.description = 'Add the channels on the list_of_channels to the given organization. This command asks interactively for the minimum data required to create each organization and later the user can update the information using the frontend or the get and set subcommands.';
AddChannel.examples = [`$ kyso channel add <organization> <list_of_channels>`];
AddChannel.args = [
    {
        name: 'organization',
        required: true,
        description: 'Organization name',
    },
    {
        name: 'list_of_channels',
        description: 'List of channels separated by commas',
        required: true,
    },
];
AddChannel.flags = {
    yes: core_1.Flags.boolean({
        char: 'y',
        description: "Don't ask with prompter and use default values",
        required: false,
        default: false,
    }),
};
