"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const kyso_model_1 = require("@kyso-io/kyso-model");
const kyso_store_1 = require("@kyso-io/kyso-store");
const jwt_decode_1 = tslib_1.__importDefault(require("jwt-decode"));
const helper_1 = require("../../helpers/helper");
const interactive_login_1 = require("../../helpers/interactive-login");
const kyso_command_1 = require("../kyso-command");
class DeleteChannel extends kyso_command_1.KysoCommand {
    async deleteChannel(api, tokenPermissions, organizationSlug, channelSlug) {
        const indexOrganization = tokenPermissions.organizations.findIndex((resourcePermissionOrganization) => resourcePermissionOrganization.name === organizationSlug);
        if (indexOrganization === -1 && !helper_1.Helper.isGlobalAdmin(tokenPermissions)) {
            this.log(`You don't belong to the organization '${organizationSlug}'`);
            return;
        }
        api.setOrganizationSlug(organizationSlug);
        const organizationResourcePermissions = tokenPermissions.organizations[indexOrganization];
        let team = null;
        try {
            const teamResponse = await api.getTeamBySlug(organizationResourcePermissions.id, channelSlug);
            team = teamResponse.data;
        }
        catch (e) {
            const errorResponse = e.response.data;
            if (errorResponse.statusCode === 404) {
                this.log(`Error: The channel '${channelSlug}' doesn't exist`);
                return;
            }
        }
        const indexTeam = tokenPermissions.teams.findIndex((resourcePermissionTeam) => resourcePermissionTeam.name === channelSlug);
        if (indexTeam === -1) {
            this.log(`You don't belong to the channel '${channelSlug}'`);
            return;
        }
        api.setTeamSlug(channelSlug);
        const teamResourcePermissions = tokenPermissions.teams[indexTeam];
        const isGlobalAdmin = helper_1.Helper.isGlobalAdmin(tokenPermissions);
        const isOrgAdmin = helper_1.Helper.isOrganizationAdmin(organizationResourcePermissions);
        const hasPermissionTeamAdmin = helper_1.Helper.hasPermission(teamResourcePermissions, kyso_model_1.TeamPermissionsEnum.ADMIN);
        const hasPermissionTeamDelete = helper_1.Helper.hasPermission(teamResourcePermissions, kyso_model_1.TeamPermissionsEnum.DELETE);
        if (!isGlobalAdmin && !isOrgAdmin && !hasPermissionTeamAdmin && !hasPermissionTeamDelete) {
            this.log(`You don't have permissions to delete the channel ${channelSlug} from the organization ${organizationSlug}`);
            return;
        }
        try {
            await api.deleteTeam(team.id);
            this.log(`Channel '${channelSlug}' deleted`);
        }
        catch (e) {
            this.log(`Error deleting the organization '${organizationSlug}' ${e.response.data.message}`);
        }
    }
    async run() {
        const { args } = await this.parse(DeleteChannel);
        await (0, interactive_login_1.launchInteractiveLoginIfNotLogged)();
        const kysoCredentials = kyso_command_1.KysoCommand.getCredentials();
        const result = await helper_1.Helper.getOrganizationFromSlugSecurely(args.organization, kysoCredentials);
        const slugifiedOrganization = result.data.sluglified_name;
        const channelsSlugs = await helper_1.Helper.getRealChannelsSlugFromStringArray(result.data, args.list_of_channels, kysoCredentials);
        const decoded = (0, jwt_decode_1.default)(kysoCredentials.token);
        const api = new kyso_store_1.Api();
        api.configure(`${kysoCredentials.kysoInstallUrl}/api/v1`, kysoCredentials === null || kysoCredentials === void 0 ? void 0 : kysoCredentials.token);
        let tokenPermissions = null;
        try {
            const resultPermissions = await api.getUserPermissions(decoded.payload.username);
            tokenPermissions = resultPermissions.data;
        }
        catch (e) {
            this.error('Error getting user permissions');
        }
        for (const channelSlug of channelsSlugs) {
            await this.deleteChannel(api, tokenPermissions, slugifiedOrganization, channelSlug);
        }
    }
}
exports.default = DeleteChannel;
DeleteChannel.description = 'Remove the channels on the list_of_channels from the given organization.';
DeleteChannel.examples = [`$ kyso organization del <organization> <list_of_channels>`];
DeleteChannel.args = [
    {
        name: 'organization',
        description: 'Organization name',
        required: true,
    },
    {
        name: 'list_of_channels',
        description: 'List of channels separated by commas',
        required: true,
    },
];
