"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const kyso_store_1 = require("@kyso-io/kyso-store");
const fs_1 = require("fs");
const jsYaml = tslib_1.__importStar(require("js-yaml"));
const jwt_decode_1 = tslib_1.__importDefault(require("jwt-decode"));
const path_1 = require("path");
const helper_1 = require("../../helpers/helper");
const interactive_login_1 = require("../../helpers/interactive-login");
const kyso_command_1 = require("../kyso-command");
class ChannelsGet extends kyso_command_1.KysoCommand {
    async getChannelData(api, tokenPermissions, organization, channelSlug) {
        let channelData = null;
        const index = tokenPermissions.teams.findIndex((resourcePermissionTeam) => resourcePermissionTeam.name === channelSlug);
        if (index === -1) {
            this.log(`You don't have permissions to get the information for the channel '${channelSlug}'`);
            return channelData;
        }
        let team = null;
        try {
            const resultTeam = await api.getTeamBySlug(organization.id, channelSlug);
            team = resultTeam.data;
            channelData = {
                organization: organization.sluglified_name,
                slug: team.sluglified_name,
                visibility: team.visibility,
                display_name: team.display_name,
                description: team.bio,
            };
        }
        catch (e) {
            if (e.response.status === 404) {
                this.log(`Error: Channel '${channelSlug}' not found`);
                return channelData;
            }
            this.log(`An error occurred getting the channel '${channelSlug}': ${e.response.data.message}`);
            return channelData;
        }
        return channelData;
    }
    async run() {
        const { args } = await this.parse(ChannelsGet);
        if (!args.yaml_file.endsWith('.yaml') && !args.yaml_file.endsWith('.yml')) {
            this.error('Yaml file name must end with .yaml or .yml');
        }
        await (0, interactive_login_1.launchInteractiveLoginIfNotLogged)();
        const kysoCredentials = kyso_command_1.KysoCommand.getCredentials();
        const secureOrg = await helper_1.Helper.getOrganizationFromSlugSecurely(args.organization, kysoCredentials);
        const slugifiedOrganization = secureOrg.data.sluglified_name;
        const channelsNames = await helper_1.Helper.getRealChannelsSlugFromStringArray(secureOrg.data, args.list_of_channels, kysoCredentials);
        const api = new kyso_store_1.Api();
        api.configure(`${kysoCredentials.kysoInstallUrl}/api/v1`, kysoCredentials === null || kysoCredentials === void 0 ? void 0 : kysoCredentials.token);
        const decoded = (0, jwt_decode_1.default)(kysoCredentials.token);
        let tokenPermissions = null;
        try {
            const resultPermissions = await api.getUserPermissions(decoded.payload.username);
            tokenPermissions = resultPermissions.data;
        }
        catch (e) {
            this.error('Error getting user permissions');
        }
        const indexOrganization = tokenPermissions.organizations.findIndex((resourcePermissionOrganization) => resourcePermissionOrganization.name === slugifiedOrganization);
        if (indexOrganization === -1 && !helper_1.Helper.isGlobalAdmin(tokenPermissions)) {
            this.error(`You don't belong to the organization '${args.organization}'`);
        }
        let organization = null;
        try {
            const resultOrganization = await api.getOrganizationBySlug(slugifiedOrganization);
            organization = resultOrganization.data;
        }
        catch (e) {
            if (e.response.status === 404) {
                this.error(`Organization '${args.organization}' not found`);
            }
            this.error(`${e.response.data.message}`);
        }
        const result = [];
        for (const channelName of channelsNames) {
            const channelData = await this.getChannelData(api, tokenPermissions, organization, channelName);
            if (!channelData) {
                continue;
            }
            result.push(channelData);
        }
        if (result.length === 0) {
            this.log('No channels found');
            return;
        }
        const yamlData = jsYaml.dump(result);
        const yamlFilePath = (0, path_1.resolve)(args.yaml_file);
        (0, fs_1.writeFileSync)(yamlFilePath, yamlData);
        this.log(`Channels data saved in ${yamlFilePath}`);
    }
}
exports.default = ChannelsGet;
ChannelsGet.description = 'Save the configuration values of the channels on the list_of_channels from the given organization on a yaml_file.';
ChannelsGet.examples = [`$ kyso organization get <organization> <list_of_channels> <yaml_file>`];
ChannelsGet.args = [
    {
        name: 'organization',
        description: 'Organization name',
        required: true,
    },
    {
        name: 'list_of_channels',
        description: 'List of channels separated by commas',
        required: true,
    },
    {
        name: 'yaml_file',
        description: 'Yaml file where the channel data will be saved',
        required: true,
    },
];
