"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const kyso_model_1 = require("@kyso-io/kyso-model");
const kyso_store_1 = require("@kyso-io/kyso-store");
const fs_1 = require("fs");
const jsYaml = tslib_1.__importStar(require("js-yaml"));
const jwt_decode_1 = tslib_1.__importDefault(require("jwt-decode"));
const helper_1 = require("../../helpers/helper");
const interactive_login_1 = require("../../helpers/interactive-login");
const kyso_command_1 = require("../kyso-command");
class ChannelsSet extends kyso_command_1.KysoCommand {
    async updateChannel(api, tokenPermissions, channelData) {
        if (!channelData.organization) {
            this.log(`Error: field 'organization' is required`);
            return;
        }
        if (!channelData.slug) {
            this.log(`Error: field 'slug' is required`);
            return;
        }
        let organization = null;
        try {
            const resultOrganization = await api.getOrganizationBySlug(channelData.organization);
            organization = resultOrganization.data;
        }
        catch (e) {
            const errorResponse = e.response.data;
            if (errorResponse.statusCode === 404) {
                this.log(`Error: Organization '${channelData.organization}' does not exist`);
                return;
            }
            this.log(`An error occurred getting the organization '${channelData.organization}'`);
            return;
        }
        api.setOrganizationSlug(organization.sluglified_name);
        const indexOrganization = tokenPermissions.organizations.findIndex((resourcePermissionOrganization) => resourcePermissionOrganization.name === channelData.organization);
        if (indexOrganization === -1 && !helper_1.Helper.isGlobalAdmin(tokenPermissions)) {
            this.log(`You don't belong to the organization ${channelData.organization}`);
            return;
        }
        let channel = null;
        try {
            const channelResponse = await api.getTeamBySlug(organization.id, channelData.slug);
            channel = channelResponse.data;
        }
        catch (e) {
            const errorResponse = e.response.data;
            if (errorResponse.statusCode === 404) {
                this.log(`Error: Channel '${channelData.slug}' does not exist`);
                return;
            }
            this.log(`An error occurred getting the channel '${channelData.slug}'`);
            return;
        }
        const indexChannel = tokenPermissions.teams.findIndex((resourcePermissionChannel) => resourcePermissionChannel.organization_id === organization.id && resourcePermissionChannel.name === channelData.slug);
        if (indexChannel === -1) {
            this.log(`You don't belong to the channel ${channelData.slug} of the organization ${channelData.organization}`);
            return;
        }
        try {
            const checkPermissionDto = new kyso_model_1.CheckPermissionDto(channelData.organization, channelData.slug, kyso_model_1.TeamPermissionsEnum.EDIT);
            const checkPermissionsResponse = await api.checkPermission(checkPermissionDto);
            if (!checkPermissionsResponse.data) {
                this.log(`You don't have permissions to edit the information for the channel '${channelData.slug}' of the organization '${channelData.organization}'`);
                return;
            }
        }
        catch (e) {
            this.error('');
        }
        const updateChannelDto = {};
        if (channelData.display_name && channelData.display_name !== channel.display_name) {
            updateChannelDto.display_name = channelData.display_name;
        }
        if (channelData.visibility && channelData.visibility !== channel.visibility) {
            updateChannelDto.visibility = channelData.visibility;
        }
        if (channelData.description && channelData.description !== channel.bio) {
            updateChannelDto.bio = channelData.description;
        }
        if (Object.keys(updateChannelDto).length > 0) {
            try {
                api.setTeamSlug(channel.sluglified_name);
                await api.updateTeam(channel.id, updateChannelDto);
                this.log(`Channel '${channel.sluglified_name}' updated`);
            }
            catch (e) {
                this.log(`Error updating channel '${channelData.slug}': ${e.response.data.message}`);
            }
        }
        else {
            this.log(`Channel '${channelData.slug}' is already up to date`);
        }
    }
    async run() {
        const { args } = await this.parse(ChannelsSet);
        if (!args.yaml_file.endsWith('.yaml') && !args.yaml_file.endsWith('.yml')) {
            this.error('File is not a yaml file');
        }
        if (!(0, fs_1.existsSync)(args.yaml_file)) {
            this.log(`File ${args.yaml_file} does not exist`);
            return;
        }
        // Read yaml file
        const yamlFileContent = (0, fs_1.readFileSync)(args.yaml_file, 'utf8');
        let channelsData = null;
        try {
            channelsData = jsYaml.load(yamlFileContent);
            if (!Array.isArray(channelsData)) {
                this.error('File is not an array of channels');
            }
        }
        catch (e) {
            this.error('File is not a valid yaml file');
        }
        await (0, interactive_login_1.launchInteractiveLoginIfNotLogged)();
        const kysoCredentials = kyso_command_1.KysoCommand.getCredentials();
        const api = new kyso_store_1.Api();
        api.configure(`${kysoCredentials.kysoInstallUrl}/api/v1`, kysoCredentials === null || kysoCredentials === void 0 ? void 0 : kysoCredentials.token);
        const decoded = (0, jwt_decode_1.default)(kysoCredentials.token);
        let tokenPermissions = null;
        try {
            const resultPermissions = await api.getUserPermissions(decoded.payload.username);
            tokenPermissions = resultPermissions.data;
        }
        catch (e) {
            this.error('Error getting user permissions');
        }
        for (const channelData of channelsData) {
            if (!channelData.slug) {
                continue;
            }
            // Re-slug just in case
            channelData.slug = helper_1.Helper.slug(channelData.slug);
            await this.updateChannel(api, tokenPermissions, channelData);
        }
    }
}
exports.default = ChannelsSet;
ChannelsSet.description = 'Pass the channels configuration values from the yaml_file to the backend and merge its values with the existing ones.\nThis command does not get the organization or the channel names because we can pass multiple channels on the YAML file and each of them includes the channel name and the organization it belongs to.';
ChannelsSet.examples = [`$ kyso channel set <yaml_file>`];
ChannelsSet.args = [
    {
        name: 'yaml_file',
        description: 'Yaml file with channels data',
        required: true,
    },
];
