"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const kyso_model_1 = require("@kyso-io/kyso-model");
const kyso_store_1 = require("@kyso-io/kyso-store");
const core_1 = require("@oclif/core");
const adm_zip_1 = tslib_1.__importDefault(require("adm-zip"));
const fs_1 = require("fs");
const path_1 = require("path");
const helper_1 = require("../helpers/helper");
const interactive_login_1 = require("../helpers/interactive-login");
const kyso_command_1 = require("./kyso-command");
class Clone extends kyso_command_1.KysoCommand {
    async run() {
        const { flags } = await this.parse(Clone);
        if (flags.verbose) {
            this.log('Enabled verbose mode');
            this.enableVerbose();
        }
        const parsed = await this.parse(Clone);
        const { cloneUrl } = parsed.args;
        if (!cloneUrl) {
            this.log('\nError: Must provide the report URL\n');
            return;
        }
        const urlParts = cloneUrl.replace('https://', '').replace('http://', '').split('/');
        const organizationSlug = urlParts[1];
        const teamSlug = urlParts[2];
        const reportSlug = urlParts[3];
        if (!organizationSlug || !teamSlug || !reportSlug) {
            this.error('Invalid report URL');
        }
        let baseUrl = cloneUrl.replace(`/${organizationSlug}/${teamSlug}/${reportSlug}`, '');
        if (baseUrl.endsWith('/')) {
            baseUrl = baseUrl.slice(0, -1);
        }
        const kysoCredentials = kyso_command_1.KysoCommand.getCredentials();
        const api = new kyso_store_1.Api();
        api.configure(`${baseUrl}/api/v1`, kysoCredentials === null || kysoCredentials === void 0 ? void 0 : kysoCredentials.token);
        let organization = null;
        try {
            const resultOrganization = await api.getOrganizationBySlug(organizationSlug);
            organization = resultOrganization.data;
        }
        catch (error) {
            const errorResponse = error.response.data;
            if (errorResponse.statusCode === 404) {
                this.log(`\nError: Organization ${organizationSlug} does not exist.\n`);
            }
            else {
                this.log(`\nError: ${errorResponse.message}.\n`);
            }
            return;
        }
        // Check if team is public
        try {
            const resultTeam = await api.getTeamBySlug(organization.id, teamSlug);
            const team = resultTeam.data;
            if (team.visibility !== kyso_model_1.TeamVisibilityEnum.PUBLIC) {
                await (0, interactive_login_1.launchInteractiveLoginIfNotLogged)();
            }
        }
        catch (error) {
            const errorResponse = error.response.data;
            if (errorResponse.statusCode === 404) {
                this.log(`\nError: Channel ${teamSlug} does not exist.\n`);
            }
            else if (errorResponse.statusCode === 403) {
                if (kysoCredentials === null || kysoCredentials === void 0 ? void 0 : kysoCredentials.token) {
                    this.log(`\nError: ${errorResponse.message}\n`);
                }
                else {
                    await (0, interactive_login_1.launchInteractiveLoginIfNotLogged)();
                    this.run();
                }
            }
            else {
                helper_1.Helper.printErrorMessage(error);
            }
            return;
        }
        this.log(`\n✨✨✨ Cloning ${cloneUrl} ✨✨✨\n`);
        try {
            let files = (0, fs_1.readdirSync)(flags.path);
            if (organizationSlug && teamSlug && reportSlug) {
                await this.extractReport(baseUrl, organizationSlug, teamSlug, reportSlug, flags.version, flags.path);
            }
            else {
                if (flags === null || flags === void 0 ? void 0 : flags.path) {
                    files = files.map((file) => (0, path_1.join)(flags.path, file));
                }
                const { kysoConfigFile, valid, message } = helper_1.Helper.findKysoConfigFile(files);
                if (!valid) {
                    this.error(`Could not clone report using Kyso config file: ${message}`);
                }
                this.extractReport(baseUrl, kysoConfigFile.organization, kysoConfigFile.team, kysoConfigFile.title, flags.version, flags.path);
            }
        }
        catch (error) {
            try {
                const errorResponse = JSON.parse(error.response.data.toString());
                this.log(`Error: ${errorResponse.message}`);
            }
            catch {
                helper_1.Helper.printErrorMessage(error);
            }
        }
        if (flags.verbose) {
            this.log('Disabling verbose mode');
            this.disableVerbose();
        }
    }
    async extractReport(baseUrl, organization, team, report, version, path) {
        var _a;
        const api = new kyso_store_1.Api();
        api.configure(`${baseUrl}/api/v1`, (_a = kyso_command_1.KysoCommand.getCredentials()) === null || _a === void 0 ? void 0 : _a.token, organization, team);
        const finalPath = `${path}/${report}`;
        const result = await api.pullReport(report, team, version);
        const zip = new adm_zip_1.default(result);
        zip.extractAllTo(finalPath, true);
        this.log(`\n🎉🎉🎉 Success! Report downloaded to ${(0, path_1.resolve)(finalPath)} 🎉🎉🎉\n`);
    }
}
exports.default = Clone;
Clone.description = 'Clone a report from Kyso';
Clone.examples = [`$ kyso clone <report_url>`];
Clone.flags = {
    path: core_1.Flags.string({
        char: 'p',
        description: 'Destination folder in which the report will be pulled',
        required: false,
        default: '.',
    }),
    version: core_1.Flags.integer({
        char: 'v',
        description: 'Version of the report to be pulled. Latest version is pulled if not set',
        required: false,
    }),
    verbose: core_1.Flags.boolean({
        char: 'x',
        description: 'Verbose mode for debugging',
        required: false,
        default: false,
    }),
};
Clone.args = [{ name: 'cloneUrl' }];
