"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable max-depth */
/* eslint-disable no-prototype-builtins */
/* eslint-disable no-negated-condition */
const core_1 = require("@oclif/core");
const fs_1 = require("fs");
const path_1 = require("path");
const uuid_1 = require("uuid");
const inquirer = require("inquirer");
const kyso_command_1 = require("./kyso-command");
const helper_1 = require("../helpers/helper");
class Format extends kyso_command_1.KysoCommand {
    async run() {
        const { flags } = await this.parse(Format);
        if (flags.verbose) {
            this.log('Enabled verbose mode');
            this.enableVerbose();
        }
        if (flags.jupyter) {
            this.formatJupyterFiles(flags);
        }
        if (flags.verbose) {
            this.log('Disabling verbose mode');
            this.disableVerbose();
        }
    }
    async formatJupyterFiles(flags) {
        let _yes = false;
        if (!flags.yes) {
            const result = await inquirer.prompt([
                {
                    type: 'confirm',
                    name: 'inlineComments',
                    message: `Jupyter notebooks of v4.5 & above have unique cell identifiers, allowing Kyso to add inline comments to the reports. It seems that you are using an older version of Jupyter. If you want to allow for inline comments on your report without updating your version of Jupyter, select 'yes' and Kyso will process all notebooks in this push & set a random identifier automatically to all cells, with no side effects to the content of the report(s). If you select 'no' the notebooks will be published without cell ids.          

These changes will modify your .ipynb files in your local filesystem, do you want to continue?\n`,
                    default: true,
                },
            ]);
            _yes = result.inlineComments;
        }
        else {
            _yes = true;
        }
        const _files = helper_1.Helper.getValidFiles(flags.path);
        const hasIpynbFiles = _files.some((x) => x.path.includes('.ipynb'));
        if (!hasIpynbFiles) {
            this.log(`Can't find any .ipynb files in the base path ${flags.path}`);
            return;
        }
        if (_yes) {
            const basePath = (0, path_1.isAbsolute)(flags.path) ? flags.path : (0, path_1.join)('.', flags.path);
            let files = helper_1.Helper.getAllFiles(basePath, []);
            const { kysoConfigFile, valid, message } = helper_1.Helper.findKysoConfigFile(files);
            if (!valid) {
                this.error(`Could not format the report: ${message}`);
            }
            if (kysoConfigFile === null || kysoConfigFile === void 0 ? void 0 : kysoConfigFile.reports) {
                for (const reportFolderName of kysoConfigFile.reports) {
                    const folderBasePath = (0, path_1.join)(basePath, reportFolderName);
                    const folderFiles = helper_1.Helper.getValidFiles(folderBasePath);
                    files = [...files, ...folderFiles.map((x) => x.path)];
                }
            }
            else {
                const a = helper_1.Helper.getValidFiles(basePath);
                files = a.map((x) => x.path);
            }
            for (const file of files) {
                if (file.endsWith('.ipynb') && !file.includes('ipynb_checkpoints')) {
                    this.log(`🔍 Processing ${file}...`);
                    let modifiedFile = false;
                    const fileContent = JSON.parse((0, fs_1.readFileSync)(file, 'utf8').toString());
                    let identifiersAdded = 0;
                    for (const cell of fileContent.cells) {
                        if (!cell.hasOwnProperty('id')) {
                            cell.id = (0, uuid_1.v4)().slice(0, 8);
                            modifiedFile = true;
                            identifiersAdded++;
                        }
                    }
                    if (modifiedFile) {
                        this.log(`⚙️  Added ${identifiersAdded} identifiers. Using git? Remember to push your changes! `);
                        (0, fs_1.writeFileSync)(file, JSON.stringify(fileContent, null, 2));
                        this.log(`💾 Saved ${file}`);
                    }
                }
            }
        }
    }
}
exports.default = Format;
Format.description = 'Format your current report files to add new great features';
Format.examples = [`$ kyso format --jupyter`];
Format.flags = {
    path: core_1.Flags.string({
        char: 'p',
        description: 'Path to root folder of the report to push. Default is "."',
        required: false,
        default: '.',
    }),
    jupyter: core_1.Flags.boolean({
        char: 'j',
        description: 'Search and process all the ipynb files to allow inline comments at Kyso',
        required: false,
    }),
    yes: core_1.Flags.boolean({
        char: 'y',
        description: 'Automatically answers "yes" to all the prompts',
        required: false,
    }),
    verbose: core_1.Flags.boolean({
        char: 'x',
        description: 'Verbose mode for debugging',
        required: false,
        default: false,
    }),
};
Format.args = [];
