"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const kyso_model_1 = require("@kyso-io/kyso-model");
const kyso_store_1 = require("@kyso-io/kyso-store");
const core_1 = require("@oclif/core");
const fs_1 = require("fs");
const jsYaml = tslib_1.__importStar(require("js-yaml"));
const jwt_decode_1 = tslib_1.__importDefault(require("jwt-decode"));
const path_1 = require("path");
const inquirer = require("inquirer");
const interactive_login_1 = require("../helpers/interactive-login");
const kyso_command_1 = require("./kyso-command");
const helper_1 = require("../helpers/helper");
var ReportTypes;
(function (ReportTypes) {
    ReportTypes["website"] = "website";
    ReportTypes["jupyter"] = "jupyter";
    ReportTypes["markdown"] = "markdown";
    ReportTypes["meta"] = "meta";
    ReportTypes["other"] = "other";
})(ReportTypes || (ReportTypes = {}));
class Init extends kyso_command_1.KysoCommand {
    async run() {
        const { flags } = await this.parse(Init);
        if (flags.verbose) {
            this.log('Enabled verbose mode');
            this.enableVerbose();
        }
        await (0, interactive_login_1.launchInteractiveLoginIfNotLogged)();
        if (!(0, fs_1.existsSync)(flags.path)) {
            this.error('Invalid path');
        }
        if (!(0, fs_1.lstatSync)(flags.path).isDirectory()) {
            this.error('Path must be a directory');
        }
        const basePath = (0, path_1.isAbsolute)(flags.path) ? flags.path : (0, path_1.join)('.', flags.path);
        const files = (0, fs_1.readdirSync)(basePath);
        const { kysoConfigFile } = helper_1.Helper.findKysoConfigFile(files);
        if (kysoConfigFile) {
            const confirmResponse = await inquirer.prompt([
                {
                    name: 'confirmOverwrite',
                    default: false,
                    message: 'kyso.yaml already exists, overwrite?',
                    type: 'confirm',
                },
            ]);
            if (!confirmResponse.confirmOverwrite) {
                return;
            }
        }
        const kysoCredentials = kyso_command_1.KysoCommand.getCredentials();
        const decoded = (0, jwt_decode_1.default)(kysoCredentials.token);
        const api = new kyso_store_1.Api(kysoCredentials.token);
        let tokenPermissions = null;
        try {
            const resultPermissions = await api.getUserPermissions(decoded.payload.username);
            tokenPermissions = resultPermissions.data;
        }
        catch (e) {
            this.error('Error getting user permissions');
        }
        if (tokenPermissions.organizations.length === 0) {
            this.error('You need to be part of an organization to initialize a report');
        }
        const organizationsPermissions = tokenPermissions.organizations.filter((organizationResourcePermission) => {
            const organizationHasCreateReportPermission = helper_1.Helper.hasPermission(organizationResourcePermission, kyso_model_1.ReportPermissionsEnum.CREATE);
            if (organizationHasCreateReportPermission) {
                return true;
            }
            const teamsOrganizationResourcePermissions = [];
            for (const teamResourcePermission of tokenPermissions.teams) {
                if (teamResourcePermission.organization_id !== organizationResourcePermission.id) {
                    continue;
                }
                if (teamResourcePermission.organization_inherited && organizationHasCreateReportPermission) {
                    teamsOrganizationResourcePermissions.push(teamResourcePermission);
                }
                else if (helper_1.Helper.hasPermission(teamResourcePermission, kyso_model_1.ReportPermissionsEnum.CREATE)) {
                    teamsOrganizationResourcePermissions.push(teamResourcePermission);
                }
            }
            return teamsOrganizationResourcePermissions.length > 0;
        });
        const organizationResponse = await inquirer.prompt([
            {
                name: 'organization',
                message: 'Select an organization',
                type: 'list',
                choices: organizationsPermissions.map((resourcePermission) => ({ name: resourcePermission.name })),
            },
        ]);
        const organizationResourcePermission = organizationsPermissions.find((resourcePermission) => resourcePermission.name === organizationResponse.organization);
        const organizationHasCreateReportPermission = helper_1.Helper.hasPermission(organizationResourcePermission, kyso_model_1.ReportPermissionsEnum.CREATE);
        const teamsOrganizationResourcePermissions = [];
        for (const teamResourcePermission of tokenPermissions.teams) {
            if (teamResourcePermission.organization_id !== organizationResourcePermission.id) {
                continue;
            }
            if (teamResourcePermission.organization_inherited && organizationHasCreateReportPermission) {
                teamsOrganizationResourcePermissions.push(teamResourcePermission);
            }
            else if (helper_1.Helper.hasPermission(teamResourcePermission, kyso_model_1.ReportPermissionsEnum.CREATE)) {
                teamsOrganizationResourcePermissions.push(teamResourcePermission);
            }
        }
        const teamResponse = await inquirer.prompt([
            {
                name: 'team',
                message: 'Select a channel',
                type: 'list',
                choices: teamsOrganizationResourcePermissions.map((teamResourcePermissions) => ({ name: teamResourcePermissions.name })),
            },
        ]);
        const reportTypeResponse = await inquirer.prompt([
            {
                name: 'reportType',
                message: 'Select a reportType',
                type: 'list',
                choices: [{ name: ReportTypes.website }, { name: ReportTypes.jupyter }, { name: ReportTypes.markdown }, { name: ReportTypes.meta }, { name: ReportTypes.other }],
            },
        ]);
        const defaultFile = '';
        /* https://gitlab.kyso.io/kyso-io/qa/issues/-/issues/134
        let defaultFile = 'index.html'
        if (reportTypeResponse.reportType === ReportTypes.jupyter) {
          defaultFile = 'index.ipynb'
        } else if (reportTypeResponse.reportType === ReportTypes.markdown) {
          defaultFile = 'index.md'
        } */
        const mainFileResponse = await inquirer.prompt([
            {
                name: 'mainFile',
                message: 'Entrypoint file?',
                type: 'input',
                default: defaultFile,
                validate(mainFile) {
                    if (mainFile === '') {
                        return 'main file cannot be empty';
                    }
                    return true;
                },
                filter: (input) => input.trim(),
            },
        ]);
        const titleResponse = await inquirer.prompt([
            {
                name: 'title',
                message: 'Title',
                type: 'input',
                default: (0, path_1.basename)(process.cwd()),
                validate(title) {
                    if (title === '') {
                        return 'title cannot be empty';
                    }
                    return true;
                },
                filter: (input) => input.trim(),
            },
        ]);
        const config = {
            organization: organizationResponse.organization,
            team: teamResponse.team,
            type: reportTypeResponse.reportType,
            title: titleResponse.title,
            main: mainFileResponse.mainFile,
        };
        (0, fs_1.writeFileSync)((0, path_1.join)(flags.path, 'kyso.yaml'), jsYaml.dump(config));
        this.log(`Wrote config to ${(0, path_1.join)(flags.path, 'kyso.yaml')}`);
        if (flags.verbose) {
            this.log('Disabling verbose mode');
            this.disableVerbose();
        }
    }
}
exports.default = Init;
Init.description = 'Interactivel build a kyso.yaml file';
Init.examples = [`$ kyso init`];
Init.flags = {
    path: core_1.Flags.string({
        char: 'p',
        description: "Folder's path in which kyso.json, yaml or yml file is placed",
        required: false,
        default: '.',
    }),
    verbose: core_1.Flags.boolean({
        char: 'x',
        description: 'Verbose mode for debugging',
        required: false,
        default: false,
    }),
};
Init.args = [];
