"use strict";
/* eslint-disable complexity */
Object.defineProperty(exports, "__esModule", { value: true });
const kyso_model_1 = require("@kyso-io/kyso-model");
const kyso_store_1 = require("@kyso-io/kyso-store");
const core_1 = require("@oclif/core");
const interactive_login_1 = require("../helpers/interactive-login");
const oauths_1 = require("../helpers/oauths");
const kyso_command_1 = require("./kyso-command");
class Login extends kyso_command_1.KysoCommand {
    async run() {
        const { flags } = await this.parse(Login);
        if (flags.verbose) {
            this.log('Enabled verbose mode');
            this.enableVerbose();
        }
        let loginModel = new kyso_model_1.Login('', kyso_model_1.LoginProviderEnum.KYSO, '', null);
        if ((flags === null || flags === void 0 ? void 0 : flags.provider) && flags.provider !== '') {
            // NON-INTERACTIVE MODE
            switch (flags.provider) {
                case kyso_model_1.LoginProviderEnum.KYSO: {
                    if (!flags.hasOwnProperty('kysoInstallUrl')) {
                        this.error('KysoInstallUrl is required when provider is kyso');
                    }
                    loginModel.kysoInstallUrl = flags.kysoInstallUrl;
                    if (!flags.hasOwnProperty('username')) {
                        this.error('Username is required when provider is kyso');
                    }
                    loginModel.email = flags.username;
                    if (flags.hasOwnProperty('password')) {
                        loginModel.password = flags.password;
                    }
                    else if (flags.hasOwnProperty('token')) {
                        loginModel.provider = kyso_model_1.LoginProviderEnum.KYSO_ACCESS_TOKEN;
                        loginModel.password = flags.token;
                    }
                    else {
                        this.error('You must provide a password or a token');
                    }
                    break;
                }
                case kyso_model_1.LoginProviderEnum.GOOGLE: {
                    try {
                        const googleResult = await (0, oauths_1.authenticateWithGoogle)(flags.kysoInstallUrl);
                        if (googleResult.errorMessage) {
                            throw new Error(googleResult.errorMessage);
                        }
                        loginModel = new kyso_model_1.Login(googleResult.code, kyso_model_1.LoginProviderEnum.GOOGLE, '', googleResult.redirectUrl);
                    }
                    catch (error) {
                        this.error(error);
                    }
                    break;
                }
                case kyso_model_1.LoginProviderEnum.GITHUB: {
                    const githubResult = await (0, oauths_1.authenticateWithGithub)(flags.kysoInstallUrl);
                    if (githubResult.errorMessage) {
                        throw new Error(githubResult.errorMessage);
                    }
                    loginModel = new kyso_model_1.Login(githubResult.code, kyso_model_1.LoginProviderEnum.GITHUB, '', null);
                    break;
                }
                case kyso_model_1.LoginProviderEnum.BITBUCKET: {
                    try {
                        const bitbucketResult = await (0, oauths_1.authenticateWithBitbucket)(flags.kysoInstallUrl);
                        if (bitbucketResult.errorMessage) {
                            throw new Error(bitbucketResult.errorMessage);
                        }
                        loginModel = new kyso_model_1.Login(bitbucketResult.code, kyso_model_1.LoginProviderEnum.BITBUCKET, '', null);
                    }
                    catch (error) {
                        this.error(error);
                    }
                    break;
                }
                case kyso_model_1.LoginProviderEnum.GITLAB: {
                    try {
                        const gitlabResponse = await (0, oauths_1.authenticateWithGitlab)(flags.kysoInstallUrl);
                        if (gitlabResponse.errorMessage) {
                            throw new Error(gitlabResponse.errorMessage);
                        }
                        loginModel = new kyso_model_1.Login(gitlabResponse.code, kyso_model_1.LoginProviderEnum.GITLAB, '', gitlabResponse.redirectUrl);
                    }
                    catch (error) {
                        this.error(error);
                    }
                    break;
                }
                default: {
                    this.error('Provider not supported');
                }
            }
            if (loginModel.kysoInstallUrl) {
                process.env.KYSO_API = `${loginModel.kysoInstallUrl}/api/v1`;
            }
        }
        else {
            // INTERACTIVE MODE
            try {
                loginModel = await (0, interactive_login_1.interactiveLogin)(kyso_command_1.KysoCommand.getCredentials());
            }
            catch (error) {
                this.error(error);
            }
        }
        try {
            const api = new kyso_store_1.Api();
            const loginResult = await api.login(loginModel);
            if (loginResult.data) {
                kyso_command_1.KysoCommand.saveToken(loginResult.data, flags.organization || null, flags.channel || null, loginModel.kysoInstallUrl, loginModel.email || null);
                this.log('Logged successfully');
            }
        }
        catch (error) {
            const errorResponse = error.response.data;
            this.error(`Login failed: ${errorResponse.message}`);
        }
        if (flags.verbose) {
            this.log('Disabling verbose mode');
            this.disableVerbose();
        }
    }
}
exports.default = Login;
Login.description = 'Login into Kyso';
Login.examples = [
    `** NOTE: If you plan to use kyso-cli inside a CI/CD pipeline, we strongly recommend to use access tokens **`,
    `# To use the interactive login
    $ kyso login`,
    `# Direct login using kyso provider and password
    $ kyso login --kysoInstallUrl <kyso_installation_url> --provider kyso --username <your_email> --password <your_password>`,
    `# Direct login using kyso provider and access token
    $ kyso login --kysoInstallUrl <kyso_installation_url> --provider kyso --username <your_email> --token <your_access_token>`,
    `# Login using github provider (will prompt a browser window to log in). The same behavior happens using the rest of external providers 
    $ kyso login --provider github`,
];
Login.flags = {
    provider: core_1.Flags.string({
        char: 'r',
        description: 'Authentication provider',
        required: false,
        // options: [LoginProviderEnum.KYSO, LoginProviderEnum.GOOGLE, LoginProviderEnum.GITHUB, LoginProviderEnum.BITBUCKET],
        // options: [LoginProviderEnum.KYSO, LoginProviderEnum.GOOGLE, LoginProviderEnum.GITHUB, LoginProviderEnum.GITLAB],
        options: [kyso_model_1.LoginProviderEnum.KYSO, kyso_model_1.LoginProviderEnum.GOOGLE, kyso_model_1.LoginProviderEnum.GITLAB],
    }),
    kysoInstallUrl: core_1.Flags.string({
        char: 'y',
        description: 'Url of your Kyso installation',
        required: false,
        multiple: false,
    }),
    username: core_1.Flags.string({
        char: 'u',
        description: 'Your email',
        required: false,
        multiple: false,
    }),
    token: core_1.Flags.string({
        char: 'k',
        description: 'Your access token',
        required: false,
        multiple: false,
    }),
    password: core_1.Flags.string({
        char: 'p',
        description: 'Your password',
        required: false,
        multiple: false,
    }),
    // Should we give that option? Hidden
    organization: core_1.Flags.string({
        char: 'o',
        description: 'Your organization',
        required: false,
        hidden: true,
    }),
    // Should we give that option? Hidden
    channel: core_1.Flags.string({
        char: 'c',
        description: 'Your channel',
        required: false,
        hidden: true,
    }),
    verbose: core_1.Flags.boolean({
        char: 'x',
        description: 'Verbose mode for debugging',
        required: false,
        default: false,
    }),
};
Login.args = [];
