"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const kyso_model_1 = require("@kyso-io/kyso-model");
const kyso_store_1 = require("@kyso-io/kyso-store");
const helper_1 = require("../../helpers/helper");
const interactive_login_1 = require("../../helpers/interactive-login");
const kyso_command_1 = require("../kyso-command");
class AddMemberToOrganization extends kyso_command_1.KysoCommand {
    async addUserToOrganization(api, userEmail, organizationSlug, role) {
        try {
            const kysoCredentials = kyso_command_1.KysoCommand.getCredentials();
            const organizationObject = await api.getOrganizationBySlug(organizationSlug);
            if (!organizationObject || !organizationObject.data) {
                this.error(`Provided organization ${organizationSlug} not found at ${kysoCredentials.kysoInstallUrl}`);
            }
            const usersResponse = await api.getUsers({ userIds: [], page: 1, per_page: 1000, sort: 'email', search: encodeURIComponent(userEmail) });
            if (!usersResponse || !usersResponse.data || usersResponse.data.length === 0) {
                this.error(`Provided user ${userEmail} not found at ${kysoCredentials.kysoInstallUrl}`);
            }
            const newMember = new kyso_model_1.AddUserOrganizationDto(organizationObject.data.id, usersResponse.data[0].id, role || 'team-reader');
            api.setOrganizationSlug(organizationObject.data.sluglified_name);
            await api.addUserToOrganization(newMember);
            this.log(`\n${usersResponse.data[0].display_name} added to organization ${organizationObject.data.display_name} at ${kysoCredentials.kysoInstallUrl}\n`);
        }
        catch (e) {
            if (!e.response || !e.response.data || !e.response.data.message) {
                this.log(`Error adding ${userEmail} to ${organizationSlug}: ${e.message}`);
            }
            else {
                this.log(`Error adding ${userEmail} to ${organizationSlug}: ${e.response.data.message}`);
            }
        }
    }
    async run() {
        const { args } = await this.parse(AddMemberToOrganization);
        const list_of_users = args.list_of_user_email.split(',');
        const organizationName = args.organization_name;
        const desiredRole = args.role;
        await (0, interactive_login_1.launchInteractiveLoginIfNotLogged)();
        const kysoCredentials = kyso_command_1.KysoCommand.getCredentials();
        const api = new kyso_store_1.Api();
        api.configure(`${kysoCredentials.kysoInstallUrl}/api/v1`, kysoCredentials === null || kysoCredentials === void 0 ? void 0 : kysoCredentials.token);
        for (const userEmail of list_of_users) {
            try {
                await this.addUserToOrganization(api, userEmail.toLowerCase(), helper_1.Helper.slug(organizationName), desiredRole);
            }
            catch (e) {
                this.log(`Error adding user ${userEmail} to org ${organizationName} with role ${desiredRole}`);
                console.log(e);
            }
        }
    }
}
exports.default = AddMemberToOrganization;
AddMemberToOrganization.description = 'Add [LIST_OF_USER_EMAIL] users as member to [ORGANIZATION_NAME] with the desired role [ROLE].';
AddMemberToOrganization.examples = [
    `# Add users with default role (team-reader)`,
    `$ kyso organization add-member lo+rey@dev.kyso.io,lo+palpatine@dev.kyso.io darkside`,
    `# Add users with specific role`,
    `$ kyso organization add-member lo+rey@dev.kyso.io,lo+palpatine@dev.kyso.io darkside organization-admin`,
];
AddMemberToOrganization.args = [
    {
        name: 'list_of_user_email',
        description: 'Emails of the users to be added to the organization separated by commas.\nExample: lo+rey@dev.kyso.io,lo+palpatine@dev.kyso.io',
        required: true,
    },
    {
        name: 'organization_name',
        description: 'Organization slug name in which the user would be added.\nExample: darkside',
        required: true,
    },
    {
        name: 'role',
        description: '<optional argument> Default is team-reader.\nExample: organization-admin',
        required: false,
        options: ['team-reader', 'team-contributor', 'team-admin', 'organization-admin'],
    },
];
