"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const kyso_model_1 = require("@kyso-io/kyso-model");
const kyso_store_1 = require("@kyso-io/kyso-store");
const jwt_decode_1 = tslib_1.__importDefault(require("jwt-decode"));
const inquirer = require("inquirer");
const helper_1 = require("../../helpers/helper");
const interactive_login_1 = require("../../helpers/interactive-login");
const kyso_command_1 = require("../kyso-command");
class AddOrganizations extends kyso_command_1.KysoCommand {
    async createOrganization(api, organizationDisplayName) {
        const createOrganizationDto = new kyso_model_1.CreateOrganizationDto(organizationDisplayName, '', '', '', kyso_model_1.AllowDownload.ALL);
        const locationResponse = await inquirer.prompt([
            {
                type: 'input',
                name: 'location',
                message: `What is the location of the organization '${createOrganizationDto.display_name}'?`,
            },
        ]);
        createOrganizationDto.location = locationResponse.location;
        const linkResponse = await inquirer.prompt([
            {
                type: 'input',
                name: 'link',
                message: `What is the link of the organization '${createOrganizationDto.display_name}'?`,
                validate(link) {
                    if (link) {
                        // Check if link is vaild url
                        const urlRegex = new RegExp('^(https?:\\/\\/)?' + // protocol
                            '((([a-z\\d]([a-z\\d-]*[a-z\\d])*)\\.)+[a-z]{2,}|' + // domain name
                            '((\\d{1,3}\\.){3}\\d{1,3}))' + // OR ip (v4) address
                            '(\\:\\d+)?(\\/[-a-z\\d%_.~+]*)*' + // port and path
                            '(\\?[;&a-z\\d%_.~+=-]*)?' + // query string
                            '(\\#[-a-z\\d_]*)?$', 'i'); // fragment locator
                        if (!urlRegex.test(link)) {
                            return 'Link is not a valid url';
                        }
                    }
                    return true;
                },
            },
        ]);
        createOrganizationDto.link = linkResponse.link;
        const bioResponse = await inquirer.prompt([
            {
                type: 'input',
                name: 'bio',
                message: `What is the bio of the organization '${createOrganizationDto.display_name}'?`,
            },
        ]);
        createOrganizationDto.bio = bioResponse.bio;
        try {
            const result = await api.createOrganization(createOrganizationDto);
            const organization = result.data;
            const kysoCredentials = kyso_command_1.KysoCommand.getCredentials();
            this.log(`\nOrganization '${organization.display_name}' created. Visit its page ${kysoCredentials.kysoInstallUrl}/${organization.sluglified_name}\n`);
        }
        catch (e) {
            this.log(`Error creating the organization: ${e.response.data.message}`);
        }
    }
    async run() {
        const { args } = await this.parse(AddOrganizations);
        const organizationsNames = args.list_of_orgs.split(',');
        await (0, interactive_login_1.launchInteractiveLoginIfNotLogged)();
        const kysoCredentials = kyso_command_1.KysoCommand.getCredentials();
        const api = new kyso_store_1.Api();
        api.configure(`${kysoCredentials.kysoInstallUrl}/api/v1`, kysoCredentials === null || kysoCredentials === void 0 ? void 0 : kysoCredentials.token);
        try {
            const value = await api.getSettingValue(kyso_model_1.KysoSettingsEnum.ONLY_GLOBAL_ADMINS_CAN_CREATE_ORGANIZATIONS);
            if (value.data === 'true' || value.data === true) {
                const decoded = (0, jwt_decode_1.default)(kysoCredentials.token);
                try {
                    const resultPermissions = await api.getUserPermissions(decoded.payload.username);
                    const tokenPermissions = resultPermissions.data;
                    const isGlobalAdmin = helper_1.Helper.isGlobalAdmin(tokenPermissions);
                    if (!isGlobalAdmin) {
                        this.log('You are not allowed to create organizations. Please contact your system administrator.');
                        return;
                    }
                }
                catch (e) {
                    this.log('Error getting user permissions');
                    return;
                }
            }
        }
        catch (e) {
            this.log('You are not allowed to create organizations. Please contact your system administrator.');
            return;
        }
        for (const organizationDisplayName of organizationsNames) {
            await this.createOrganization(api, organizationDisplayName);
        }
    }
}
exports.default = AddOrganizations;
AddOrganizations.description = 'Add the organizations in the list_of_orgs to the system. This command asks interactively for the minimum data required to create them and later the user can update the information using the frontend or the get and set subcommands.';
AddOrganizations.examples = [`$ kyso organization add <list_of_orgs>`];
AddOrganizations.args = [
    {
        name: 'list_of_orgs',
        description: 'List of organizations separated by commas',
        required: true,
    },
];
