"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const kyso_model_1 = require("@kyso-io/kyso-model");
const kyso_store_1 = require("@kyso-io/kyso-store");
const jwt_decode_1 = tslib_1.__importDefault(require("jwt-decode"));
const helper_1 = require("../../helpers/helper");
const interactive_login_1 = require("../../helpers/interactive-login");
const kyso_command_1 = require("../kyso-command");
class DeleteOrganization extends kyso_command_1.KysoCommand {
    async deleteOrganization(api, tokenPermissions, organizationSlug) {
        const indexOrganization = tokenPermissions.organizations.findIndex((resourcePermissionOrganization) => resourcePermissionOrganization.name === organizationSlug);
        if (indexOrganization === -1 && !helper_1.Helper.isGlobalAdmin(tokenPermissions)) {
            this.log(`You don't have permissions to delete the organization '${organizationSlug}'`);
            return;
        }
        api.setOrganizationSlug(organizationSlug);
        const resourcePermissions = tokenPermissions.organizations[indexOrganization];
        const hasPermissionDelete = helper_1.Helper.hasPermission(resourcePermissions, kyso_model_1.OrganizationPermissionsEnum.DELETE);
        const isOrgAdmin = helper_1.Helper.isOrganizationAdmin(resourcePermissions);
        const isGlobalAdmin = helper_1.Helper.isGlobalAdmin(tokenPermissions);
        if (!hasPermissionDelete && !isOrgAdmin && !isGlobalAdmin) {
            this.log(`You don't have permissions to delete the organization '${organizationSlug}'`);
            return;
        }
        let organizationId = null;
        if (!resourcePermissions) {
            try {
                const organizationResult = await api.getOrganizationBySlug(organizationSlug);
                organizationId = organizationResult.data.id;
            }
            catch (e) {
                this.log(`Error getting the organization '${organizationSlug}': ${e.response.data.message}`);
                return;
            }
        }
        else {
            organizationId = resourcePermissions.id;
        }
        try {
            await api.deleteOrganization(organizationId);
            this.log(`Organization '${organizationSlug}' deleted`);
        }
        catch (e) {
            this.log(`Error deleting the organization '${organizationSlug}': ${e.response.data.message}`);
        }
    }
    async run() {
        const { args } = await this.parse(DeleteOrganization);
        await (0, interactive_login_1.launchInteractiveLoginIfNotLogged)();
        const kysoCredentials = kyso_command_1.KysoCommand.getCredentials();
        // Slug the organization to ensure that if someone introduced the name of the organization in
        // capital letters we are going to be able to answer properly
        const organizationsSlugs = await helper_1.Helper.getRealOrganizationSlugFromStringArray(args.list_of_orgs, kysoCredentials);
        const decoded = (0, jwt_decode_1.default)(kysoCredentials.token);
        const api = new kyso_store_1.Api();
        api.configure(`${kysoCredentials.kysoInstallUrl}/api/v1`, kysoCredentials === null || kysoCredentials === void 0 ? void 0 : kysoCredentials.token);
        let tokenPermissions = null;
        try {
            const resultPermissions = await api.getUserPermissions(decoded.payload.username);
            tokenPermissions = resultPermissions.data;
        }
        catch (e) {
            this.error('Error getting user permissions');
        }
        for (const organizationSlug of organizationsSlugs) {
            await this.deleteOrganization(api, tokenPermissions, organizationSlug);
        }
    }
}
exports.default = DeleteOrganization;
DeleteOrganization.description = 'Remove the organizations included in the list_of_orgs.';
DeleteOrganization.examples = [`$ kyso organization del <list_of_orgs>`];
DeleteOrganization.args = [
    {
        name: 'list_of_orgs',
        description: 'List of organizations separated by commas',
        required: true,
    },
];
