"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const kyso_store_1 = require("@kyso-io/kyso-store");
const core_1 = require("@oclif/core");
const axios_1 = tslib_1.__importDefault(require("axios"));
const fs_1 = require("fs");
const jsYaml = tslib_1.__importStar(require("js-yaml"));
const jwt_decode_1 = tslib_1.__importDefault(require("jwt-decode"));
const path_1 = require("path");
const helper_1 = require("../../helpers/helper");
const interactive_login_1 = require("../../helpers/interactive-login");
const kyso_command_1 = require("../kyso-command");
class OrganizationsGet extends kyso_command_1.KysoCommand {
    async getOrganizationData(api, tokenPermissions, organizationSlug, getChannels, getImages) {
        var _a, _b, _c, _d;
        let organizationData = null;
        const indexOrganization = tokenPermissions.organizations.findIndex((resourcePermissionOrganization) => resourcePermissionOrganization.name === organizationSlug);
        const isGlobalAdmin = helper_1.Helper.isGlobalAdmin(tokenPermissions);
        if (indexOrganization === -1 && !isGlobalAdmin) {
            this.log(`You don't have permissions to get the information for the organization '${organizationSlug}'`);
            return organizationData;
        }
        const resourcePermissions = tokenPermissions.organizations[indexOrganization];
        const isOrgAdmin = helper_1.Helper.isOrganizationAdmin(resourcePermissions);
        if (!isOrgAdmin && !isGlobalAdmin) {
            this.log(`You don't have permissions to get the information for the organization '${organizationSlug}'`);
            return organizationData;
        }
        const kysoCredentials = kyso_command_1.KysoCommand.getCredentials();
        let organization = null;
        try {
            const resultOrganization = await api.getOrganizationBySlug(organizationSlug);
            organization = resultOrganization.data;
            organizationData = {
                slug: organization.sluglified_name,
                display_name: organization.display_name,
                allowed_access_domains: organization.allowed_access_domains,
                location: organization.location,
                link: organization.link,
                bio: organization.bio,
                channels: [],
                options: {
                    /* auth: {
                      otherProviders: organization.options?.auth?.otherProviders || [],
                    }, */
                    notifications: {
                        centralized: ((_b = (_a = organization.options) === null || _a === void 0 ? void 0 : _a.notifications) === null || _b === void 0 ? void 0 : _b.centralized) || false,
                        emails: ((_d = (_c = organization.options) === null || _c === void 0 ? void 0 : _c.notifications) === null || _d === void 0 ? void 0 : _d.emails) || [],
                    },
                },
            };
            if (getChannels) {
                api.setOrganizationSlug(organizationSlug);
                const resultTeams = await api.getTeams({
                    page: 1,
                    per_page: 10000,
                    filter: {
                        organization_id: organization.id,
                    },
                });
                const teams = resultTeams.data;
                organizationData.channels = teams.map((team) => team.sluglified_name);
            }
            if (getImages) {
                if (organization.avatar_url) {
                    try {
                        const imageUrl = organization.avatar_url.startsWith('http') ? organization.avatar_url : kysoCredentials.kysoInstallUrl + organization.avatar_url;
                        const axiosResponse = await axios_1.default.get(imageUrl, { responseType: 'text', responseEncoding: 'base64' });
                        if (axiosResponse.status === 200) {
                            organizationData.photo = `data:${axiosResponse.headers['content-type']};base64,${axiosResponse.data}`;
                        }
                    }
                    catch (e) {
                        console.log(e);
                    }
                }
            }
            return organizationData;
        }
        catch (e) {
            return organizationData;
        }
    }
    async run() {
        const { args, flags } = await this.parse(OrganizationsGet);
        if (!args.yaml_file.endsWith('.yaml') && !args.yaml_file.endsWith('.yml')) {
            this.error('Yaml file name must end with .yaml or .yml');
        }
        await (0, interactive_login_1.launchInteractiveLoginIfNotLogged)();
        const kysoCredentials = kyso_command_1.KysoCommand.getCredentials();
        // Slug the organization to ensure that if someone introduced the name of the organization in
        // capital letters we are going to be able to answer properly
        const organizationsSlugs = await helper_1.Helper.getRealOrganizationSlugFromStringArray(args.list_of_orgs, kysoCredentials);
        const api = new kyso_store_1.Api();
        api.configure(`${kysoCredentials.kysoInstallUrl}/api/v1`, kysoCredentials === null || kysoCredentials === void 0 ? void 0 : kysoCredentials.token);
        const decoded = (0, jwt_decode_1.default)(kysoCredentials.token);
        let tokenPermissions = null;
        try {
            const resultPermissions = await api.getUserPermissions(decoded.payload.username);
            tokenPermissions = resultPermissions.data;
        }
        catch (e) {
            this.error('Error getting user permissions');
        }
        const result = [];
        for (const organizationSlug of organizationsSlugs) {
            const organizationData = await this.getOrganizationData(api, tokenPermissions, organizationSlug, flags.channels, flags.images);
            if (!organizationData) {
                continue;
            }
            result.push(organizationData);
        }
        if (result.length === 0) {
            this.log('No organizations found');
            return;
        }
        const yamlData = jsYaml.dump(result);
        const yamlFilePath = (0, path_1.resolve)(args.yaml_file);
        (0, fs_1.writeFileSync)(yamlFilePath, yamlData);
        this.log(`Organizations data saved in ${yamlFilePath}`);
    }
}
exports.default = OrganizationsGet;
OrganizationsGet.description = 'Save the configuration values (profile data, access settings and list of channels) of the organizations in the comma_separated_list_of_orgs on a yaml_file.\nIf the --images flag is passed the <yaml_file> will contain copies of the background and photo images encoded in base64.\nIf the --no-channels flag is passed the <yaml_file> will not include the list of channels for the organizations.';
OrganizationsGet.examples = [
    `$ kyso organization get <list_of_orgs> <yaml_file>`,
    `$ kyso organization get <list_of_orgs> <yaml_file> --images`,
    `$ kyso organization get <list_of_orgs> <yaml_file> --no-channels`,
    `$ kyso organization get <list_of_orgs> <yaml_file> --images --no-channels`,
];
OrganizationsGet.flags = {
    images: core_1.Flags.boolean({
        char: 'i',
        description: 'Get organization image in base64 format',
        required: false,
        default: false,
    }),
    channels: core_1.Flags.boolean({
        char: 'c',
        description: 'Get organization channels',
        required: false,
        default: true,
        allowNo: true,
    }),
};
OrganizationsGet.args = [
    {
        name: 'list_of_orgs',
        description: 'List of organizations separated by commas',
        required: true,
    },
    {
        name: 'yaml_file',
        description: 'Yaml file where the organization data will be saved',
        required: true,
    },
];
