"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const kyso_model_1 = require("@kyso-io/kyso-model");
const kyso_store_1 = require("@kyso-io/kyso-store");
const fs_1 = require("fs");
const jwt_decode_1 = tslib_1.__importDefault(require("jwt-decode"));
const helper_1 = require("../../../helpers/helper");
const interactive_login_1 = require("../../../helpers/interactive-login");
const kyso_command_1 = require("../../kyso-command");
class UploadPhoto extends kyso_command_1.KysoCommand {
    async run() {
        const { args } = await this.parse(UploadPhoto);
        // Check if file exists
        if (!(0, fs_1.existsSync)(args.image_file)) {
            this.log(`File ${args.image_file} does not exist`);
            return;
        }
        // Check if file is an image
        if (!helper_1.Helper.isImage(args.image_file)) {
            this.log(`File ${args.image_file} is not an image. Valid formats are: png, jpg, jpeg, gif`);
            return;
        }
        await (0, interactive_login_1.launchInteractiveLoginIfNotLogged)();
        const kysoCredentials = kyso_command_1.KysoCommand.getCredentials();
        const organization = await helper_1.Helper.getOrganizationFromSlugSecurely(args.organization, kysoCredentials);
        const api = new kyso_store_1.Api();
        api.configure(`${kysoCredentials.kysoInstallUrl}/api/v1`, kysoCredentials === null || kysoCredentials === void 0 ? void 0 : kysoCredentials.token, organization.data.sluglified_name);
        const decoded = (0, jwt_decode_1.default)(kysoCredentials.token);
        let tokenPermissions = null;
        try {
            const resultPermissions = await api.getUserPermissions(decoded.payload.username);
            tokenPermissions = resultPermissions.data;
        }
        catch (e) {
            this.error('Error getting user permissions');
        }
        const indexOrganization = tokenPermissions.organizations.findIndex((resourcePermissionOrganization) => resourcePermissionOrganization.name === organization.data.sluglified_name);
        if (indexOrganization === -1 && !helper_1.Helper.isGlobalAdmin(tokenPermissions)) {
            this.log(`You don't have permissions to upload the photo for the organization ${args.organization}`);
            return;
        }
        const resourcePermissions = tokenPermissions.organizations[indexOrganization];
        const hasPermissionDelete = helper_1.Helper.hasPermission(resourcePermissions, kyso_model_1.OrganizationPermissionsEnum.DELETE);
        const isOrgAdmin = helper_1.Helper.isOrganizationAdmin(resourcePermissions);
        const isGlobalAdmin = helper_1.Helper.isGlobalAdmin(tokenPermissions);
        if (!hasPermissionDelete && !isOrgAdmin && !isGlobalAdmin) {
            this.log(`You don't have permissions to upload the photo for the organization ${args.organization}`);
            return;
        }
        try {
            const readStream = (0, fs_1.createReadStream)(args.image_file);
            await api.uploadOrganizationImage(resourcePermissions.id, readStream);
            this.log(`Photo uploaded successfully`);
        }
        catch (e) {
            this.log(`Error uploading image: ${e.response.data.message}`);
        }
    }
}
exports.default = UploadPhoto;
UploadPhoto.description = 'Upload the image_file as the `photo` image of the given organization';
UploadPhoto.examples = [`$ kyso organization upload photo <organization> <image_file>`];
UploadPhoto.args = [
    {
        name: 'organization',
        description: 'Organization name',
        required: true,
    },
    {
        name: 'image_file',
        required: true,
        description: 'Path to the image',
    },
];
