"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const kyso_store_1 = require("@kyso-io/kyso-store");
const core_1 = require("@oclif/core");
const axios_1 = tslib_1.__importDefault(require("axios"));
const fs_1 = require("fs");
const jsYaml = tslib_1.__importStar(require("js-yaml"));
const jwt_decode_1 = tslib_1.__importDefault(require("jwt-decode"));
const path_1 = require("path");
const interactive_login_1 = require("../../helpers/interactive-login");
const kyso_command_1 = require("../kyso-command");
class UserProfileGet extends kyso_command_1.KysoCommand {
    async run() {
        const { args, flags } = await this.parse(UserProfileGet);
        if (!args.yaml_file.endsWith('.yaml') && !args.yaml_file.endsWith('.yml')) {
            this.error('Yaml file name must end with .yaml or .yml');
        }
        await (0, interactive_login_1.launchInteractiveLoginIfNotLogged)();
        const kysoCredentials = kyso_command_1.KysoCommand.getCredentials();
        const api = new kyso_store_1.Api();
        api.configure(`${kysoCredentials.kysoInstallUrl}/api/v1`, kysoCredentials === null || kysoCredentials === void 0 ? void 0 : kysoCredentials.token);
        const decoded = (0, jwt_decode_1.default)(kysoCredentials.token);
        let userDTO = null;
        let profileData = null;
        try {
            const result = await api.getUserProfileByUsername(decoded.payload.username);
            userDTO = result.data;
            profileData = {
                email: userDTO.email,
                username: userDTO.username,
                name: userDTO.name,
                display_name: userDTO.display_name,
                bio: userDTO.bio,
                location: userDTO.location,
                link: userDTO.link,
            };
        }
        catch (e) {
            this.error(`Error getting user profile: ${e.response.data.message}`);
        }
        if (flags.images) {
            if (userDTO.avatar_url) {
                try {
                    const imageUrl = userDTO.avatar_url.startsWith('http') ? userDTO.avatar_url : kysoCredentials.kysoInstallUrl + userDTO.avatar_url;
                    const axiosResponse = await axios_1.default.get(imageUrl, { responseType: 'text', responseEncoding: 'base64' });
                    if (axiosResponse.status === 200) {
                        profileData.photo = `data:${axiosResponse.headers['content-type']};base64,${axiosResponse.data}`;
                    }
                }
                catch (e) {
                    console.log(e);
                }
            }
            if (userDTO.background_image_url) {
                try {
                    const backgroundImageUrl = userDTO.background_image_url.startsWith('http') ? userDTO.background_image_url : kysoCredentials.kysoInstallUrl + userDTO.background_image_url;
                    const axiosResponse = await axios_1.default.get(backgroundImageUrl, { responseType: 'text', responseEncoding: 'base64' });
                    if (axiosResponse.status === 200) {
                        profileData.background = `data:${axiosResponse.headers['content-type']};base64,${axiosResponse.data}`;
                    }
                }
                catch (e) {
                    console.log(e);
                }
            }
        }
        const yamlString = jsYaml.dump(profileData);
        const path = (0, path_1.resolve)(args.yaml_file);
        (0, fs_1.writeFileSync)(path, yamlString);
        this.log(`User profile data saved in ${path}`);
    }
}
exports.default = UserProfileGet;
UserProfileGet.description = 'Save the user profile values on a yaml_file.\nIf the --images flag is passed the yaml_file will contain copies of the background and photo images encoded in base64.';
UserProfileGet.examples = [`$ kyso profile get <yaml_file>`];
UserProfileGet.flags = {
    images: core_1.Flags.boolean({
        char: 'i',
        description: 'Get user profile images in base64 format',
        required: false,
        default: false,
    }),
};
UserProfileGet.args = [
    {
        name: 'yaml_file',
        description: 'Yaml file name where the user profile data will be saved',
        required: true,
    },
];
