"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const kyso_store_1 = require("@kyso-io/kyso-store");
const axios_1 = tslib_1.__importDefault(require("axios"));
const fs_1 = require("fs");
const jsYaml = tslib_1.__importStar(require("js-yaml"));
const jwt_decode_1 = tslib_1.__importDefault(require("jwt-decode"));
const path_1 = require("path");
const interactive_login_1 = require("../../helpers/interactive-login");
const kyso_command_1 = require("../kyso-command");
class UserProfileSet extends kyso_command_1.KysoCommand {
    async updatePhoto(api, userId, photoBase64) {
        if (!photoBase64.startsWith('data:image/')) {
            this.error('Photo must be a base64 string');
        }
        try {
            // Create file from base64 string
            const formatImage = photoBase64.split(';')[0].split('/')[1];
            const imageFilePath = (0, path_1.join)(kyso_command_1.KysoCommand.DATA_DIRECTORY, `photo-${new Date().getTime()}.${formatImage}`);
            const buffer = Buffer.from(photoBase64.split(',')[1], 'base64');
            (0, fs_1.writeFileSync)(imageFilePath, buffer);
            const readStream = (0, fs_1.createReadStream)(imageFilePath);
            await api.uploadUserProfileImage(userId, readStream);
            (0, fs_1.unlinkSync)(imageFilePath);
        }
        catch (e) {
            this.log(`Error uploading profile photo: ${e.response.data.message}`);
        }
    }
    async updateBackgroundImage(api, userId, backgroundBase64) {
        if (!backgroundBase64.startsWith('data:image/')) {
            this.error('Background image must be a base64 string');
        }
        try {
            // Create file from base64 string
            const formatImage = backgroundBase64.split(';')[0].split('/')[1];
            const imageFilePath = (0, path_1.join)(kyso_command_1.KysoCommand.DATA_DIRECTORY, `background-${new Date().getTime()}.${formatImage}`);
            const buffer = Buffer.from(backgroundBase64.split(',')[1], 'base64');
            (0, fs_1.writeFileSync)(imageFilePath, buffer);
            const readStream = (0, fs_1.createReadStream)(imageFilePath);
            await api.uploadUserBackgroundImage(userId, readStream);
            (0, fs_1.unlinkSync)(imageFilePath);
        }
        catch (e) {
            this.log(`Error uploading background profile image: ${e.response.data.message}`);
        }
    }
    async run() {
        const { args } = await this.parse(UserProfileSet);
        if (!args.yaml_file.endsWith('.yaml') && !args.yaml_file.endsWith('.yml')) {
            this.error('File is not a yaml file');
        }
        if (!(0, fs_1.existsSync)(args.yaml_file)) {
            this.log(`File ${args.yaml_file} does not exist`);
            return;
        }
        // Read yaml file
        const yamlFileContent = (0, fs_1.readFileSync)(args.yaml_file, 'utf8');
        let yamlProfileData = null;
        try {
            yamlProfileData = jsYaml.load(yamlFileContent);
        }
        catch (e) {
            this.error('File is not a valid yaml file');
        }
        await (0, interactive_login_1.launchInteractiveLoginIfNotLogged)();
        const kysoCredentials = kyso_command_1.KysoCommand.getCredentials();
        const api = new kyso_store_1.Api();
        api.configure(`${kysoCredentials.kysoInstallUrl}/api/v1`, kysoCredentials === null || kysoCredentials === void 0 ? void 0 : kysoCredentials.token);
        const decoded = (0, jwt_decode_1.default)(kysoCredentials.token);
        let userDto = null;
        let profileData = null;
        let photoBase64 = null;
        let backgroundImageBase64 = null;
        try {
            const result = await api.getUserProfileByUsername(decoded.payload.username);
            userDto = result.data;
            profileData = {
                email: userDto.email,
                username: userDto.username,
                name: userDto.name,
                display_name: userDto.display_name,
                bio: userDto.bio,
                location: userDto.location,
                link: userDto.link,
            };
        }
        catch (e) {
            this.error(`Error getting user profile: ${e.response.data.message}`);
        }
        if (userDto.avatar_url) {
            try {
                const imageUrl = userDto.avatar_url.startsWith('http') ? userDto.avatar_url : kysoCredentials.kysoInstallUrl + userDto.avatar_url;
                const axiosResponse = await axios_1.default.get(imageUrl, { responseType: 'text', responseEncoding: 'base64' });
                if (axiosResponse.status === 200) {
                    photoBase64 = `data:${axiosResponse.headers['content-type']};base64,${axiosResponse.data}`;
                }
            }
            catch (e) {
                console.log(e);
            }
        }
        if (userDto.background_image_url) {
            try {
                const backgroundImageUrl = userDto.background_image_url.startsWith('http') ? userDto.background_image_url : kysoCredentials.kysoInstallUrl + userDto.background_image_url;
                const axiosResponse = await axios_1.default.get(backgroundImageUrl, { responseType: 'text', responseEncoding: 'base64' });
                if (axiosResponse.status === 200) {
                    backgroundImageBase64 = `data:${axiosResponse.headers['content-type']};base64,${axiosResponse.data}`;
                }
            }
            catch (e) {
                console.log(e);
            }
        }
        let updatedPhoto = false;
        if ((!photoBase64 && yamlProfileData.photo) || (photoBase64 && yamlProfileData.photo && photoBase64 !== yamlProfileData.photo)) {
            await this.updatePhoto(api, userDto.id, yamlProfileData.photo);
            updatedPhoto = true;
        }
        let updatedBackgroundImage = false;
        if ((!backgroundImageBase64 && yamlProfileData.background) || (backgroundImageBase64 && yamlProfileData.background && backgroundImageBase64 !== yamlProfileData.background)) {
            await this.updateBackgroundImage(api, userDto.id, yamlProfileData.background);
            updatedBackgroundImage = true;
        }
        const updateUserRequestDto = {};
        if (yamlProfileData.name && yamlProfileData.name !== profileData.name) {
            updateUserRequestDto.name = yamlProfileData.name;
        }
        if (yamlProfileData.display_name && yamlProfileData.display_name !== profileData.display_name) {
            updateUserRequestDto.display_name = yamlProfileData.display_name;
        }
        if (yamlProfileData.location && yamlProfileData.location !== profileData.location) {
            updateUserRequestDto.location = yamlProfileData.location;
        }
        if (yamlProfileData.link && yamlProfileData.link !== profileData.link) {
            updateUserRequestDto.link = yamlProfileData.link;
        }
        if (yamlProfileData.bio && yamlProfileData.bio !== profileData.bio) {
            updateUserRequestDto.bio = yamlProfileData.bio;
        }
        if (Object.keys(updateUserRequestDto).length > 0) {
            try {
                await api.updateUser(userDto.id, updateUserRequestDto);
                this.log('User profile updated');
            }
            catch (e) {
                this.error(`Error updating user profile: ${e.response.data.message}`);
            }
        }
        else if (!updatedPhoto && !updatedBackgroundImage) {
            this.log('No changes to update');
        }
        else {
            this.log('User profile updated');
        }
    }
}
exports.default = UserProfileSet;
UserProfileSet.description = 'Pass the user profile values from the yaml_file to the backend and merge its values with the existing ones (that is, missing values keep their value).\nIf we pass the images encoded in base64 on the yaml_file the images are updated if there are changes.';
UserProfileSet.examples = [`$ kyso profile set <yaml_file>`];
UserProfileSet.args = [
    {
        name: 'yaml_file',
        description: 'Yaml file with user profile data',
        required: true,
    },
];
