"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const kyso_model_1 = require("@kyso-io/kyso-model");
const kyso_store_1 = require("@kyso-io/kyso-store");
const core_1 = require("@oclif/core");
const adm_zip_1 = tslib_1.__importDefault(require("adm-zip"));
const fs_1 = require("fs");
const path_1 = require("path");
const helper_1 = require("../helpers/helper");
const interactive_login_1 = require("../helpers/interactive-login");
const kyso_command_1 = require("./kyso-command");
class Push extends kyso_command_1.KysoCommand {
    async run() {
        const { flags } = await this.parse(Push);
        if (flags.verbose) {
            this.log('Enabled verbose mode');
            this.enableVerbose();
        }
        let organizationSlug = null;
        let teamSlug = null;
        let reportSlug = null;
        let kysoConfigFile = null;
        try {
            this.log('Pulling report. Please wait...');
            if ((flags === null || flags === void 0 ? void 0 : flags.organization) && (flags === null || flags === void 0 ? void 0 : flags.channel) && (flags === null || flags === void 0 ? void 0 : flags.report)) {
                organizationSlug = flags.organization;
                teamSlug = flags.channel;
                reportSlug = flags.report;
            }
            else {
                if (!flags.path) {
                    this.error('Please specify a path to pull the report to');
                }
                let files = (0, fs_1.readdirSync)(flags.path);
                files = files.map((file) => (0, path_1.join)(flags.path, file));
                const resultSearch = helper_1.Helper.findKysoConfigFile(files);
                if (!resultSearch.valid) {
                    this.error(`Could not pull report using Kyso config file: ${resultSearch.message}`);
                }
                kysoConfigFile = resultSearch.kysoConfigFile;
                organizationSlug = kysoConfigFile.organization;
                teamSlug = kysoConfigFile.channel;
                reportSlug = helper_1.Helper.slug(kysoConfigFile.title);
            }
        }
        catch (error) {
            helper_1.Helper.printErrorMessage(error);
        }
        if (!organizationSlug) {
            this.error('Organization is required');
        }
        if (!teamSlug) {
            this.error('Channel is required');
        }
        if (!reportSlug) {
            this.error('Report is required');
        }
        // Check if channel is public
        const kysoCredentials = kyso_command_1.KysoCommand.getCredentials();
        const api = new kyso_store_1.Api();
        api.configure(`${kysoCredentials === null || kysoCredentials === void 0 ? void 0 : kysoCredentials.kysoInstallUrl}/api/v1`, kysoCredentials === null || kysoCredentials === void 0 ? void 0 : kysoCredentials.token);
        let organization = null;
        try {
            const resultOrganization = await api.getOrganizationBySlug(organizationSlug);
            organization = resultOrganization.data;
        }
        catch {
            this.log(`\nError: Organization ${organizationSlug} does not exist.\n`);
            return;
        }
        api.setOrganizationSlug(organizationSlug);
        try {
            const resultTeam = await api.getTeamBySlug(organization.id, teamSlug);
            const team = resultTeam.data;
            if (team.visibility !== kyso_model_1.TeamVisibilityEnum.PUBLIC) {
                await (0, interactive_login_1.launchInteractiveLoginIfNotLogged)();
            }
        }
        catch (error) {
            const errorResponse = error.response.data;
            if (errorResponse.statusCode === 404) {
                this.log(`\nError: Channel ${teamSlug} does not exist.\n`);
            }
            else if (errorResponse.statusCode === 403) {
                if (kysoCredentials === null || kysoCredentials === void 0 ? void 0 : kysoCredentials.token) {
                    this.log(`\nError: ${errorResponse.message}\n`);
                }
                else {
                    await (0, interactive_login_1.launchInteractiveLoginIfNotLogged)();
                    this.run();
                }
            }
            else {
                helper_1.Helper.printErrorMessage(error);
            }
            return;
        }
        await this.extractReport(organizationSlug, teamSlug, reportSlug, flags.version, flags.path, kysoConfigFile);
        if (flags.verbose) {
            this.log('Disabling verbose mode');
            this.disableVerbose();
        }
    }
    async extractReport(organization, team, report, version, path, kysoConfigFile) {
        try {
            const kysoCredentials = kyso_command_1.KysoCommand.getCredentials();
            const api = new kyso_store_1.Api();
            api.configure(`${kysoCredentials === null || kysoCredentials === void 0 ? void 0 : kysoCredentials.kysoInstallUrl}/api/v1`, kysoCredentials === null || kysoCredentials === void 0 ? void 0 : kysoCredentials.token, organization, team);
            const result = await api.pullReport(report, team, version);
            const zip = new adm_zip_1.default(result);
            let finalPath = path;
            if (!kysoConfigFile) {
                finalPath = (0, path_1.join)(path, report);
            }
            zip.extractAllTo(finalPath, true);
            this.log(`\n🎉🎉🎉 Success! Report downloaded to ${(0, path_1.resolve)(finalPath)} 🎉🎉🎉\n`);
        }
        catch (error) {
            const { message } = JSON.parse(error.response.data.toString());
            this.error(message);
        }
    }
}
exports.default = Push;
Push.description = 'Pull repository from Kyso';
Push.examples = [`$ kyso pull --path <destination_folder> --organization <organization> --channel <channel> --report <report_name> --version <version>`];
Push.flags = {
    path: core_1.Flags.string({
        char: 'p',
        description: 'Destination folder in which the report will be pulled',
        required: false,
        default: '.',
    }),
    organization: core_1.Flags.string({
        char: 'o',
        description: 'Organization slug name which the report belongs to. i.e: for organization "Kyso Inc" the organization slug is "kyso-inc".',
    }),
    channel: core_1.Flags.string({
        char: 'c',
        description: 'Channel slug name which the report belongs to. i.e: for channel "My Awesome Channel" the channel slug is "my-awesome-channel".',
    }),
    report: core_1.Flags.string({
        char: 'r',
        description: 'Report slug name to be pulled. i.e: for report with name "The Great Report" the report slug if "the-great-report"',
    }),
    version: core_1.Flags.integer({
        char: 'v',
        description: 'Version of the report to be pulled. Latest version is pulled if not set',
        required: false,
    }),
    verbose: core_1.Flags.boolean({
        char: 'x',
        description: 'Verbose mode for debugging',
        required: false,
        default: false,
    }),
};
Push.args = [];
