"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const kyso_model_1 = require("@kyso-io/kyso-model");
const kyso_store_1 = require("@kyso-io/kyso-store");
const core_1 = require("@oclif/core");
const adm_zip_1 = tslib_1.__importDefault(require("adm-zip"));
const axios_1 = tslib_1.__importDefault(require("axios"));
const form_data_1 = tslib_1.__importDefault(require("form-data"));
const fs_1 = require("fs");
const hosted_git_info_1 = tslib_1.__importDefault(require("hosted-git-info"));
const jsYaml = tslib_1.__importStar(require("js-yaml"));
const jwt_decode_1 = tslib_1.__importDefault(require("jwt-decode"));
const os_1 = require("os");
const path_1 = require("path");
const simple_git_1 = require("simple-git");
const uuid_1 = require("uuid");
const helper_1 = require("../helpers/helper");
const interactive_login_1 = require("../helpers/interactive-login");
const kyso_command_1 = require("./kyso-command");
class Push extends kyso_command_1.KysoCommand {
    getEmbeddedReportHTML(title, url) {
        return `
      <!DOCTYPE html>
      <html lang="EN" xml:lang="en">
        <head>
          <meta charset="utf-8">
          <link rel="stylesheet" href="https://fonts.googleapis.com/css?family=Roboto+Mono:400,500&amp;amp;display=swap">
          <meta http-equiv="Content-Type" content="text/html; charset=utf-8">
          <title>${title}</title>
          <script type="text/javascript"></script>
        </head>
        <body>
          <iframe title="${title}" id="theframe" style="width: 100%; height: 950px; border: none;" sandbox="allow-scripts allow-same-origin allow-forms allow-modals allow-popups" src="${url}"></iframe>
          <script type="text/javascript">
            function onInitIFrame() {
              try {
                const myIframe = document.getElementById('theframe');
                setTimeout(() => {
                    setHeight(myIframe.contentWindow.document.body.scrollHeight + 20px");
                }, 1500);
              } catch (ex) {
              }
            }
          </script>
        </body>
      </html>
    `;
    }
    async getGitMetadata(localPath) {
        try {
            const options = {
                baseDir: localPath,
                binary: 'git',
                maxConcurrentProcesses: 6,
                trimmed: false,
            };
            (0, simple_git_1.simpleGit)().clean(simple_git_1.CleanOptions.FORCE);
            const git = (0, simple_git_1.simpleGit)(options);
            const logResult = await git.log();
            let repository = null;
            try {
                const remoteUrl = await git.listRemote(['--get-url']);
                const hostedGitInfoUrl = hosted_git_info_1.default.fromUrl(remoteUrl);
                repository = hostedGitInfoUrl ? helper_1.Helper.sanitizeUrlBasicAuthentication(hostedGitInfoUrl.https().replace('git+', '')) : helper_1.Helper.sanitizeUrlBasicAuthentication(remoteUrl);
            }
            catch (e) {
                // Do nothing
            }
            const branchSummary = await git.branchLocal();
            return {
                repository,
                branch: branchSummary.current,
                latest_commit: logResult.all.slice(0, 10),
            };
        }
        catch (e) {
            return null;
        }
    }
    async uploadReportAux(reportFolder, basePath, validFiles, pushMessage, metaOrganization, metaChannel, metaTags) {
        var _a;
        const kysoCredentials = kyso_command_1.KysoCommand.getCredentials();
        const api = new kyso_store_1.Api(kysoCredentials.token);
        api.configure(`${kysoCredentials.kysoInstallUrl}/api/v1`, kysoCredentials.token);
        const { kysoConfigFile, valid, message, kysoConfigPath } = helper_1.Helper.findKysoConfigFile(validFiles.map((f) => f.path), basePath);
        if (!valid) {
            this.log(`\nError: Could not push report of '${reportFolder}' folder using Kyso config file: ${message}\n`);
            return;
        }
        // SLUG everything. Is idempotent, makes no effect if already slugified. But if is the display name
        // because the user entered it by mistake, we can avoid an error slugifing by ourselves
        if (kysoConfigFile.channel) {
            kysoConfigFile.channel = helper_1.Helper.slug(kysoConfigFile.channel);
        }
        if (kysoConfigFile.team) {
            kysoConfigFile.team = helper_1.Helper.slug(kysoConfigFile.team);
        }
        let updateKysoConfigFile = false;
        // Channel not set. If there are metas, apply it
        if (!kysoConfigFile.channel && !kysoConfigFile.team) {
            if (metaChannel) {
                kysoConfigFile.team = metaChannel;
                kysoConfigFile.channel = metaChannel;
                updateKysoConfigFile = true;
            }
            else {
                // Not set and no metas ---> Error
                kysoConfigFile.team = null;
                kysoConfigFile.channel = null;
            }
        }
        if (kysoConfigFile.organization) {
            kysoConfigFile.organization = helper_1.Helper.slug(kysoConfigFile.organization);
        }
        // Organization not set. If there are metas, apply it
        if (!kysoConfigFile.organization) {
            if (metaOrganization) {
                kysoConfigFile.organization = metaOrganization;
                updateKysoConfigFile = true;
            }
            else {
                // Not set and no metas ---> Error
                kysoConfigFile.organization = null;
            }
        }
        if ((!kysoConfigFile.tags || !Array.isArray(kysoConfigFile.tags) || kysoConfigFile.tags.length === 0) && metaTags && metaTags.length > 0) {
            kysoConfigFile.tags = metaTags;
            updateKysoConfigFile = true;
        }
        const { payload } = (0, jwt_decode_1.default)(kysoCredentials.token);
        let tokenPermissions = null;
        try {
            const resultPermissions = await api.getUserPermissions(payload.username);
            tokenPermissions = resultPermissions.data;
        }
        catch (e) {
            this.error('Error getting user permissions');
        }
        const indexOrganization = tokenPermissions.organizations.findIndex((resourcePermissionOrganization) => resourcePermissionOrganization.name === kysoConfigFile.organization);
        if (indexOrganization === -1 && !helper_1.Helper.isGlobalAdmin(tokenPermissions)) {
            this.log(`\nYou don't have permissions to create reports in the '${kysoConfigFile.organization}' organization defined in the '${reportFolder}' folder.\n`);
            return;
        }
        let teamId = null;
        try {
            const checkPermissionDto = new kyso_model_1.CheckPermissionDto(kysoConfigFile.organization, kysoConfigFile.team, kyso_model_1.ReportPermissionsEnum.CREATE);
            const resultCheckPermission = await api.checkPermission(checkPermissionDto);
            if (resultCheckPermission.data) {
                const indexTeam = tokenPermissions.teams.findIndex((resourcePermissionTeam) => resourcePermissionTeam.name === kysoConfigFile.team && resourcePermissionTeam.organization_id === tokenPermissions.organizations[indexOrganization].id);
                if (indexTeam > -1) {
                    teamId = tokenPermissions.teams[indexTeam].id;
                }
                else {
                    this.log(`\You don't have permission to create reports in the '${kysoConfigFile.team}' channel of the '${kysoConfigFile.organization}' organization defined in the '${reportFolder}' folder.\n`);
                    return;
                }
            }
            else {
                this.log(`\You don't have permission to create reports in the '${kysoConfigFile.team}' channel of the '${kysoConfigFile.organization}' organization defined in the '${reportFolder}' folder.\n`);
                return;
            }
        }
        catch (error) {
            this.log(`\nError: ${error.response.data.message}\n`);
            return;
        }
        if ((kysoConfigFile === null || kysoConfigFile === void 0 ? void 0 : kysoConfigFile.organization) && kysoConfigFile.organization.length > 0) {
            kyso_store_1.store.dispatch((0, kyso_store_1.setOrganizationAuthAction)(kysoConfigFile.organization));
        }
        if ((kysoConfigFile === null || kysoConfigFile === void 0 ? void 0 : kysoConfigFile.team) && kysoConfigFile.team.length > 0) {
            kyso_store_1.store.dispatch((0, kyso_store_1.setTeamAuthAction)(kysoConfigFile.team));
        }
        delete kysoConfigFile.id;
        delete kysoConfigFile.links;
        delete kysoConfigFile.created_at;
        delete kysoConfigFile.updated_at;
        delete kysoConfigFile.team;
        let newFiles = [];
        const unmodifiedFiles = [];
        const deletedFiles = [];
        let reportDto = null;
        let version = 1;
        try {
            const reportSlug = helper_1.Helper.slug(kysoConfigFile.title);
            const resultReport = await api.getReportByTeamIdAndSlug(teamId, reportSlug);
            reportDto = resultReport.data;
            version = reportDto.last_version;
            const resultFiles = await api.getReportFiles(reportDto.id, reportDto.last_version);
            const reportFiles = resultFiles.data;
            validFiles.forEach((validFile) => {
                const indexFile = reportFiles.findIndex((reportFile) => {
                    let validFilePathWithoutBasePath = basePath === '.' ? validFile.path : validFile.path.replace(basePath, '');
                    if (validFilePathWithoutBasePath.startsWith('/')) {
                        validFilePathWithoutBasePath = validFilePathWithoutBasePath.slice(1);
                    }
                    return reportFile.sha === validFile.sha && reportFile.name === validFilePathWithoutBasePath;
                });
                if (indexFile === -1) {
                    newFiles.push(validFile.path);
                }
                else {
                    unmodifiedFiles.push(reportFiles[indexFile].id);
                }
            });
            if (newFiles.length === 0) {
                this.log(`No new or modified files to upload in report '${kysoConfigFile.title}'`);
                return;
            }
            reportFiles.forEach((reportFile) => {
                const indexFile = validFiles.findIndex((validFile) => {
                    const pathWithoutBasePath = validFile.path.replace(`${basePath}/`, '');
                    return reportFile.name === pathWithoutBasePath;
                });
                if (indexFile === -1) {
                    deletedFiles.push(reportFile.id);
                }
            });
        }
        catch (error) {
            const errorResponse = error.response.data;
            if (errorResponse.statusCode === 404) {
                newFiles = validFiles.map((file) => file.path);
            }
        }
        // Check if report has defined main file
        if (kysoConfigFile.main && validFiles.length > 0) {
            // Remove last traling / if exists
            let sanitizeBasePath = basePath.endsWith('/') ? basePath.slice(0, -1) : basePath;
            // edge case
            if (sanitizeBasePath === '.') {
                sanitizeBasePath = '';
            }
            const indexMainFile = validFiles.findIndex((x) => x.path.endsWith(`${sanitizeBasePath}/${kysoConfigFile.main}`));
            if (indexMainFile === -1) {
                this.log(`\nError: Main file '${kysoConfigFile.main}' defined in the '${reportFolder}' folder does not exist.\n`);
                return;
            }
        }
        if (!kysoConfigFile.authors || kysoConfigFile.authors.length === 0) {
            kysoConfigFile.authors = [payload.email];
            updateKysoConfigFile = true;
        }
        if ((kysoConfigFile === null || kysoConfigFile === void 0 ? void 0 : kysoConfigFile.type) === kyso_model_1.ReportType.Embedded) {
            kysoConfigFile.main = 'index.html';
            (0, fs_1.writeFileSync)(`${basePath}/index.html`, this.getEmbeddedReportHTML(kysoConfigFile.title, kysoConfigFile.url));
            updateKysoConfigFile = true;
        }
        if (updateKysoConfigFile) {
            if (kysoConfigPath.endsWith('.json')) {
                (0, fs_1.writeFileSync)(kysoConfigPath, JSON.stringify(kysoConfigFile, null, 2));
            }
            else {
                (0, fs_1.writeFileSync)(kysoConfigPath, jsYaml.dump(kysoConfigFile));
            }
        }
        const resultKysoSettings = await api.getSettingValue(kyso_model_1.KysoSettingsEnum.MAX_FILE_SIZE);
        let existsMethod = true;
        // Check the put endpoint is available in the api
        try {
            api.configure(`${kysoCredentials.kysoInstallUrl}/api/v1`, kysoCredentials.token, kysoConfigFile.organization, kysoConfigFile.team || kysoConfigFile.channel);
            await api.getHttpClient().put(`/reports/kyso/XXXX`);
        }
        catch (error) {
            const errorResponse = error.response.data;
            if (errorResponse.statusCode === 404) {
                existsMethod = false;
            }
        }
        this.log(`Uploading report '${reportFolder}'`);
        let result;
        if (existsMethod && reportDto) {
            result = await kyso_store_1.store.dispatch((0, kyso_store_1.updateKysoReportAction)({
                filePaths: newFiles,
                basePath,
                maxFileSizeStr: resultKysoSettings.data || '500mb',
                id: reportDto.id,
                unmodifiedFiles,
                deletedFiles,
                version,
                message: pushMessage,
                git_metadata: await this.getGitMetadata(basePath),
            }));
        }
        else {
            result = await kyso_store_1.store.dispatch((0, kyso_store_1.createKysoReportAction)({
                filePaths: validFiles.map((file) => file.path),
                basePath,
                maxFileSizeStr: resultKysoSettings.data || '500mb',
                message: pushMessage,
                git_metadata: await this.getGitMetadata(basePath),
            }));
        }
        const { error } = kyso_store_1.store.getState();
        if (error.text) {
            this.error(`\n😞 ${error.text}`);
        }
        if (((_a = result === null || result === void 0 ? void 0 : result.payload) === null || _a === void 0 ? void 0 : _a.isAxiosError) || result.payload === null) {
            this.error(`\n😞 Something went wrong pushing the report in '${reportFolder}' folder. Please check the console log.`);
        }
        else {
            const kysoCredentials = JSON.parse((0, fs_1.readFileSync)(kyso_command_1.KysoCommand.tokenFilePath, 'utf8').toString());
            const normalizedResponse = result.payload;
            if (Array.isArray(normalizedResponse.data)) {
                for (const reportDto of normalizedResponse.data) {
                    const reportUrl = `${kysoCredentials.kysoInstallUrl}/${reportDto.organization_sluglified_name}/${reportDto.team_sluglified_name}/${reportDto.name}`;
                    this.log(`🎉🎉🎉 Report ${reportDto.title} was uploaded to: ${reportUrl} 🎉🎉🎉\n`);
                }
            }
            else {
                const reportDto = normalizedResponse.data;
                const reportUrl = `${kysoCredentials.kysoInstallUrl}/${reportDto.organization_sluglified_name}/${reportDto.team_sluglified_name}/${reportDto.name}`;
                this.log(`🎉🎉🎉 Report ${reportDto.title} was uploaded to: ${reportUrl} 🎉🎉🎉\n`);
            }
        }
    }
    async uploadReport(validFiles, basePath, pushMessage) {
        if (validFiles.length === 0) {
            return;
        }
        // Check if report is a multiple report
        let mainKysoConfigFile = null;
        const data = helper_1.Helper.findKysoConfigFile(validFiles.map((file) => file.path), basePath);
        if (!data.valid) {
            this.log(`Error in Kyso config file: ${data.message}`);
            return;
        }
        mainKysoConfigFile = data.kysoConfigFile;
        if (mainKysoConfigFile === null || mainKysoConfigFile === void 0 ? void 0 : mainKysoConfigFile.reports) {
            this.log(`\n${mainKysoConfigFile.reports.length} ${mainKysoConfigFile.reports.length > 1 ? 'reports' : 'report'} found\n`);
            // Retrieve meta organization and meta channel (defaults)
            const metaOrganization = mainKysoConfigFile.organization;
            const metaChannel = mainKysoConfigFile.channel ? mainKysoConfigFile.channel : mainKysoConfigFile.team;
            const metaTags = mainKysoConfigFile.tags ? mainKysoConfigFile.tags : [];
            // Get report folders
            const folders = [];
            for (const reportFolder of mainKysoConfigFile.reports) {
                // Wildcard?
                if (reportFolder.endsWith('*')) {
                    const folder = reportFolder.slice(0, -1);
                    const reportPath = (0, path_1.join)(basePath, folder);
                    if (!(0, fs_1.existsSync)(reportPath)) {
                        this.error(`Folder '${reportFolder}' does not exist.`);
                    }
                    // Get folders and valid files
                    for (const fileInFolder of (0, fs_1.readdirSync)(reportPath)) {
                        const fileInFolderPath = (0, path_1.join)(reportPath, fileInFolder);
                        if ((0, fs_1.statSync)(fileInFolderPath).isDirectory()) {
                            folders.push(fileInFolder);
                        }
                        else {
                            const extension = (0, path_1.extname)(fileInFolderPath);
                            if (extension === '.md' || extension === '.ipynb') {
                                const { singleFileReports } = this.getFiles(fileInFolderPath);
                                if (singleFileReports.length === 0) {
                                    continue;
                                }
                                folders.push(fileInFolder);
                            }
                        }
                    }
                }
                else {
                    folders.push(reportFolder);
                }
            }
            for (const reportFolder of folders) {
                // Check if folder exists
                const reportPath = (0, path_1.join)(basePath, reportFolder);
                if (!(0, fs_1.existsSync)(reportPath)) {
                    this.error(`Report '${reportFolder}' folder does not exist.`);
                }
                if (!(0, fs_1.statSync)(reportPath).isDirectory()) {
                    const { singleFileReports } = this.getFiles(reportPath);
                    if (singleFileReports.length === 0) {
                        continue;
                    }
                    await this.uploadSimpleFile(basePath, singleFileReports[0], pushMessage, metaOrganization, metaChannel, metaTags);
                    continue;
                }
                const validFiles = helper_1.Helper.getValidFiles(reportPath);
                const { valid, message } = helper_1.Helper.findKysoConfigFile(validFiles.map((file) => file.path), basePath);
                if (!valid) {
                    this.error(`Folder '${reportFolder}' does not have a valid Kyso config file: ${message}`);
                }
                await this.uploadReportAux(reportFolder, reportPath, validFiles, pushMessage, metaOrganization, metaChannel, metaTags);
            }
        }
        else {
            const parts = basePath.split('/');
            const reportFolder = parts[parts.length - 1];
            await this.uploadReportAux(reportFolder, basePath, validFiles, pushMessage);
        }
    }
    getHeadersFromMarkdownFile(filePath) {
        const fileContent = (0, fs_1.readFileSync)(filePath, 'utf8');
        const lines = fileContent.split('\n');
        let startIndex = -1;
        let endIndex = -1;
        for (let i = 0; i < lines.length; i++) {
            const line = lines[i];
            if (line.startsWith('---')) {
                if (startIndex === -1) {
                    startIndex = i;
                }
                else {
                    endIndex = i;
                    break;
                }
            }
        }
        if (startIndex === -1 || endIndex === -1) {
            return null;
        }
        try {
            const headersStr = lines.slice(startIndex + 1, endIndex);
            return jsYaml.load(headersStr.join('\n'));
        }
        catch (e) {
            return null;
        }
    }
    getHeadersFromJupyterFile(filePath) {
        try {
            const fileContentStr = (0, fs_1.readFileSync)(filePath, 'utf8');
            if (!fileContentStr) {
                return null;
            }
            const fileContent = JSON.parse(fileContentStr);
            if (!fileContent.cells || !Array.isArray(fileContent.cells) || fileContent.cells.length === 0) {
                return null;
            }
            const firstCell = fileContent.cells[0];
            if (!firstCell.source || !Array.isArray(firstCell.source) || firstCell.source.length === 0) {
                return null;
            }
            const firstLine = firstCell.source[0];
            if (!firstLine.startsWith('---')) {
                return null;
            }
            const lines = firstCell.source;
            let startIndex = -1;
            let endIndex = -1;
            for (let i = 0; i < lines.length; i++) {
                const line = lines[i];
                if (line.startsWith('---')) {
                    if (startIndex === -1) {
                        startIndex = i;
                    }
                    else {
                        endIndex = i;
                        break;
                    }
                }
            }
            if (startIndex === -1 || endIndex === -1) {
                return null;
            }
            const headersStr = lines.slice(startIndex + 1, endIndex);
            return jsYaml.load(headersStr.join('\n'));
        }
        catch (e) {
            return null;
        }
    }
    getFiles(folderPath) {
        const validFiles = helper_1.Helper.getValidFiles(folderPath);
        const singleFileReports = [];
        const reportFiles = [];
        for (const validFile of validFiles) {
            const extension = (0, path_1.extname)(validFile.path);
            if (extension === '.md') {
                const headers = this.getHeadersFromMarkdownFile(validFile.path);
                if (headers) {
                    singleFileReports.push({ headers, filePath: validFile.path, sha: validFile.sha });
                }
                else {
                    reportFiles.push(validFile);
                }
            }
            else if (extension === '.ipynb') {
                const headers = this.getHeadersFromJupyterFile(validFile.path);
                if (headers) {
                    singleFileReports.push({ headers, filePath: validFile.path, sha: validFile.sha });
                }
                else {
                    reportFiles.push(validFile);
                }
            }
            else {
                reportFiles.push(validFile);
            }
        }
        if (reportFiles.length > 0) {
            const { kysoConfigFile, valid, message } = helper_1.Helper.findKysoConfigFile(validFiles.map((f) => f.path), folderPath);
            if (!valid) {
                this.log(`Error in Kyso config file: ${message}`);
                return;
            }
            const pathMainFile = (0, path_1.join)(folderPath, kysoConfigFile.main ? kysoConfigFile.main : '');
            const validMainFile = validFiles.find((f) => f.path === pathMainFile);
            const index = reportFiles.findIndex((f) => f.path === pathMainFile);
            if (validMainFile && index === -1) {
                reportFiles.push(validMainFile);
            }
        }
        return {
            singleFileReports,
            reportFiles,
        };
    }
    async uploadSimpleFile(basePath, reportSingleFile, pushMessage, metaOrganization, metaChannel, metaTags) {
        var _a;
        const kysoCredentials = kyso_command_1.KysoCommand.getCredentials();
        const api = new kyso_store_1.Api(kysoCredentials.token);
        api.configure(`${kysoCredentials.kysoInstallUrl}/api/v1`, kysoCredentials.token);
        const fileName = (0, path_1.basename)(reportSingleFile.filePath);
        const kysoConfigFile = { ...reportSingleFile.headers };
        kysoConfigFile.main = fileName;
        const { valid, message } = kyso_model_1.KysoConfigFile.isValid(kysoConfigFile);
        if (!valid) {
            this.log(`\nError: Could not push report file of '${fileName}': ${message}\n`);
            return null;
        }
        if (kysoConfigFile.channel) {
            kysoConfigFile.channel = helper_1.Helper.slug(kysoConfigFile.channel);
            kysoConfigFile.team = kysoConfigFile.channel;
        }
        else if (kysoConfigFile.team) {
            kysoConfigFile.team = helper_1.Helper.slug(kysoConfigFile.team);
            kysoConfigFile.channel = kysoConfigFile.team;
        }
        if (kysoConfigFile.organization) {
            kysoConfigFile.organization = helper_1.Helper.slug(kysoConfigFile.organization);
        }
        // Channel not set. If there are metas, apply it
        if (!kysoConfigFile.channel && !kysoConfigFile.team) {
            if (metaChannel) {
                kysoConfigFile.team = metaChannel;
                kysoConfigFile.channel = metaChannel;
            }
            else {
                // Not set and no metas ---> Error
                kysoConfigFile.team = null;
                kysoConfigFile.channel = null;
            }
        }
        // Organization not set. If there are metas, apply it
        if (!kysoConfigFile.organization) {
            if (metaOrganization) {
                kysoConfigFile.organization = metaOrganization;
            }
            else {
                // Not set and no metas ---> Error
                kysoConfigFile.organization = null;
            }
        }
        if ((!kysoConfigFile.tags || !Array.isArray(kysoConfigFile.tags) || kysoConfigFile.tags.length === 0) && metaTags && metaTags.length > 0) {
            kysoConfigFile.tags = metaTags;
        }
        if (!kysoConfigFile.organization) {
            this.log(`\nError: Organization not set in the '${reportSingleFile.filePath}' file.\n`);
            return null;
        }
        if (!kysoConfigFile.channel) {
            this.log(`\nError: Channel not set in the '${reportSingleFile.filePath}' file.\n`);
            return null;
        }
        const { payload } = (0, jwt_decode_1.default)(kysoCredentials.token);
        let tokenPermissions = null;
        try {
            const resultPermissions = await api.getUserPermissions(payload.username);
            tokenPermissions = resultPermissions.data;
        }
        catch (e) {
            this.error('Error getting user permissions');
        }
        const indexOrganization = tokenPermissions.organizations.findIndex((resourcePermissionOrganization) => resourcePermissionOrganization.name === kysoConfigFile.organization);
        if (indexOrganization === -1 && !helper_1.Helper.isGlobalAdmin(tokenPermissions)) {
            this.log(`\You don't have permissions to create reports in the '${kysoConfigFile.organization}' organization defined in the '${reportSingleFile.filePath}' file.\n`);
            return;
        }
        let teamId = null;
        try {
            const checkPermissionDto = new kyso_model_1.CheckPermissionDto(kysoConfigFile.organization, kysoConfigFile.team, kyso_model_1.ReportPermissionsEnum.CREATE);
            const resultCheckPermission = await api.checkPermission(checkPermissionDto);
            if (resultCheckPermission.data) {
                const indexTeam = tokenPermissions.teams.findIndex((resourcePermissionTeam) => resourcePermissionTeam.name === kysoConfigFile.team && resourcePermissionTeam.organization_id === tokenPermissions.organizations[indexOrganization].id);
                if (indexTeam > -1) {
                    teamId = tokenPermissions.teams[indexTeam].id;
                }
                else {
                    this.log(`\You don't have permission to create reports in the '${kysoConfigFile.team}' channel of the '${kysoConfigFile.organization}' organization defined in the '${reportSingleFile.filePath}' file.\n`);
                    return;
                }
            }
            else {
                this.log(`\You don't have permission to create reports in the '${kysoConfigFile.team}' channel of the '${kysoConfigFile.organization}' organization defined in the '${reportSingleFile.filePath}' file.\n`);
                return;
            }
        }
        catch (error) {
            this.log(`\nError: ${error.response.data.message}\n`);
            return;
        }
        if ((kysoConfigFile === null || kysoConfigFile === void 0 ? void 0 : kysoConfigFile.organization) && kysoConfigFile.organization.length > 0) {
            kyso_store_1.store.dispatch((0, kyso_store_1.setOrganizationAuthAction)(kysoConfigFile.organization));
        }
        if ((kysoConfigFile === null || kysoConfigFile === void 0 ? void 0 : kysoConfigFile.team) && kysoConfigFile.team.length > 0) {
            kyso_store_1.store.dispatch((0, kyso_store_1.setTeamAuthAction)(kysoConfigFile.team));
        }
        delete kysoConfigFile.id;
        delete kysoConfigFile.links;
        delete kysoConfigFile.created_at;
        delete kysoConfigFile.updated_at;
        delete kysoConfigFile.team;
        if (!kysoConfigFile.authors || kysoConfigFile.authors.length === 0) {
            kysoConfigFile.authors = [payload.email];
        }
        try {
            const reportSlug = helper_1.Helper.slug(kysoConfigFile.title);
            const responseReportDto = await api.getReportByTeamIdAndSlug(teamId, reportSlug);
            const reportDto = responseReportDto.data;
            const responseReportFiles = await api.getReportFiles(reportDto.id, reportDto.last_version);
            const reportFiles = responseReportFiles.data;
            const reportFile = reportFiles.find((reportFile) => reportFile.name === fileName);
            if (!reportFile) {
                this.log(`\nError: Could not find the file '${fileName}' in the report '${kysoConfigFile.title}'\n`);
                return reportDto;
            }
            if (reportFile.sha === reportSingleFile.sha) {
                this.log(`\n'${fileName}' has not changed since the last push\n`);
                return reportDto;
            }
        }
        catch (error) {
            const errorResponse = error.response.data;
            if (errorResponse.statusCode === 404) {
                // Report does not exist
            }
        }
        const zip = new adm_zip_1.default();
        zip.addFile(fileName, (0, fs_1.readFileSync)(reportSingleFile.filePath));
        zip.addFile('kyso.yaml', Buffer.from(jsYaml.dump(kysoConfigFile)));
        const zipFileName = `${(0, uuid_1.v4)()}.zip`;
        let outputFilePath = (0, path_1.join)((0, os_1.homedir)(), '.kyso', 'tmp');
        if (!(0, fs_1.existsSync)(outputFilePath)) {
            (0, fs_1.mkdirSync)(outputFilePath, {
                recursive: true,
            });
        }
        outputFilePath = (0, path_1.join)(outputFilePath, zipFileName);
        zip.writeZip(outputFilePath);
        const resultKysoSettings = await api.getSettingValue(kyso_model_1.KysoSettingsEnum.MAX_FILE_SIZE);
        const { size } = (0, fs_1.statSync)(outputFilePath);
        const maxFileSize = helper_1.Helper.parseFileSizeStr(resultKysoSettings.data || '500mb');
        if (size > maxFileSize) {
            this.log(`\nError: You exceeded the maximum upload size permitted (${payload.maxFileSizeStr})`);
            return null;
        }
        const formData = new form_data_1.default();
        formData.append('ignoreKysoConfigFile', 'true');
        formData.append('file', (0, fs_1.createReadStream)(outputFilePath), {
            filename: zipFileName,
            knownLength: size,
        });
        if (pushMessage) {
            formData.append('message', pushMessage);
        }
        const gitMetadata = await this.getGitMetadata(basePath);
        if (gitMetadata) {
            formData.append('git_metadata', JSON.stringify(payload.git_metadata));
        }
        this.log(`Uploading report '${reportSingleFile.filePath}'`);
        let reportDto = null;
        try {
            const url = `${kysoCredentials.kysoInstallUrl}/api/v1/reports/kyso`;
            const response = await axios_1.default.post(url, formData, {
                headers: {
                    ...formData.getHeaders(),
                    'content-length': formData.getLengthSync().toString(),
                    Authorization: `Bearer ${kysoCredentials.token}`,
                    'x-kyso-organization': kysoConfigFile.organization,
                    'x-kyso-team': (_a = kysoConfigFile.channel) !== null && _a !== void 0 ? _a : kysoConfigFile.team,
                },
            });
            reportDto = response.data.data;
            const reportUrl = `${kysoCredentials.kysoInstallUrl}/${reportDto.organization_sluglified_name}/${reportDto.team_sluglified_name}/${reportDto.name}`;
            this.log(`🎉🎉🎉 Report ${reportDto.title} was uploaded to: ${reportUrl} 🎉🎉🎉\n`);
        }
        catch (e) {
            const errorResponse = e.response.data;
            this.log(`\nError: ${errorResponse.message}.\n`);
        }
        finally {
            try {
                (0, fs_1.rmSync)(outputFilePath, { force: true });
            }
            catch (e) {
                this.log(`\nTemporary file can't be deleted.\n`);
            }
        }
        return reportDto;
    }
    async run() {
        const { flags } = await this.parse(Push);
        if (flags.verbose) {
            this.log('Enabled verbose mode');
            this.enableVerbose();
        }
        await (0, interactive_login_1.launchInteractiveLoginIfNotLogged)();
        let basePath = (0, path_1.isAbsolute)(flags.path) ? flags.path : (0, path_1.join)('.', flags.path);
        if (basePath.endsWith('/') || basePath.endsWith('\\')) {
            basePath = basePath.slice(0, -1);
        }
        if (!(0, fs_1.existsSync)(basePath)) {
            this.error('Invalid path');
        }
        const { singleFileReports, reportFiles } = this.getFiles(basePath);
        let uploadSingleFileReports = true;
        // Check if the user wants to upload a meta report to ignore reports with single file
        if (reportFiles.length > 0) {
            const data = helper_1.Helper.findKysoConfigFile(reportFiles.map((file) => file.path), basePath);
            if (!data.valid) {
                this.log(`Error in Kyso config file: ${data.message}`);
                return;
            }
            if (data.kysoConfigFile.reports) {
                if (!Array.isArray(data.kysoConfigFile.reports)) {
                    this.error(`\nError: 'reports' property in Kyso config file must be an array.\n`);
                }
                if (data.kysoConfigFile.reports.length === 0) {
                    this.error(`\nError: 'reports' property in Kyso config file must have at least one report.\n`);
                }
                uploadSingleFileReports = false;
            }
            await this.uploadReport(reportFiles, basePath, flags.message);
        }
        if (uploadSingleFileReports) {
            for (const reportSingleFile of singleFileReports) {
                await this.uploadSimpleFile(basePath, reportSingleFile, flags.message);
            }
        }
        if (flags.verbose) {
            this.log('Disabling verbose mode');
            this.disableVerbose();
        }
    }
}
exports.default = Push;
Push.description = 'Upload local repository to Kyso';
Push.examples = [`$ kyso push --path ./my-report`];
Push.flags = {
    path: core_1.Flags.string({
        char: 'p',
        description: 'Path to root folder of the report to push. Default is "."',
        required: false,
        default: '.',
    }),
    verbose: core_1.Flags.boolean({
        char: 'x',
        description: 'Verbose mode for debugging',
        required: false,
        default: false,
    }),
    message: core_1.Flags.string({
        char: 'm',
        description: 'Push message',
        required: false,
        default: undefined,
    }),
};
Push.args = [];
