"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const kyso_model_1 = require("@kyso-io/kyso-model");
const kyso_store_1 = require("@kyso-io/kyso-store");
const color_1 = tslib_1.__importDefault(require("@oclif/color"));
const core_1 = require("@oclif/core");
const fs_1 = require("fs");
const jwt_decode_1 = tslib_1.__importDefault(require("jwt-decode"));
const moment_1 = tslib_1.__importDefault(require("moment"));
const path_1 = require("path");
const helper_1 = require("../helpers/helper");
const interactive_login_1 = require("../helpers/interactive-login");
const kyso_command_1 = require("./kyso-command");
class Status extends kyso_command_1.KysoCommand {
    async reportStatus(reportFolder, basePath) {
        const kysoCredentials = kyso_command_1.KysoCommand.getCredentials();
        const api = new kyso_store_1.Api(kysoCredentials.token);
        api.configure(`${kysoCredentials.kysoInstallUrl}/api/v1`, kysoCredentials.token);
        const filesBasePath = (0, fs_1.readdirSync)(basePath).map((file) => (0, path_1.join)(basePath, file));
        const { kysoConfigFile, valid, message } = helper_1.Helper.findKysoConfigFile(filesBasePath);
        if (!valid) {
            this.log(`\nError: Could not pull report of '${reportFolder}' folder using Kyso config file: ${message}\n`);
            return;
        }
        const { payload } = (0, jwt_decode_1.default)(kysoCredentials.token);
        let tokenPermissions = null;
        try {
            const resultPermissions = await api.getUserPermissions(payload.username);
            tokenPermissions = resultPermissions.data;
        }
        catch (e) {
            this.error('Error getting user permissions');
        }
        const indexOrganization = tokenPermissions.organizations.findIndex((resourcePermissionOrganization) => resourcePermissionOrganization.name === kysoConfigFile.organization);
        if (indexOrganization === -1 && !helper_1.Helper.isGlobalAdmin(tokenPermissions)) {
            this.log(`\nYou don't have permissions to create reports in the '${kysoConfigFile.organization}' organization defined in the '${reportFolder}' folder.\n`);
            return;
        }
        let teamId = null;
        try {
            const checkPermissionDto = new kyso_model_1.CheckPermissionDto(kysoConfigFile.organization, kysoConfigFile.team, kyso_model_1.ReportPermissionsEnum.CREATE);
            const resultCheckPermission = await api.checkPermission(checkPermissionDto);
            if (resultCheckPermission.data) {
                const indexTeam = tokenPermissions.teams.findIndex((resourcePermissionTeam) => resourcePermissionTeam.name === kysoConfigFile.team && resourcePermissionTeam.organization_id === tokenPermissions.organizations[indexOrganization].id);
                if (indexTeam > -1) {
                    teamId = tokenPermissions.teams[indexTeam].id;
                }
                else {
                    this.log(`\nYou don't have permission to get reports in '${kysoConfigFile.team}' channel of '${kysoConfigFile.organization}' organization defined in the '${reportFolder}' folder.\n`);
                    return;
                }
            }
            else {
                this.log(`\nYou don't have permission to get reports in '${kysoConfigFile.team}' channel of '${kysoConfigFile.organization}' organization defined in the '${reportFolder}' folder.\n`);
                return;
            }
        }
        catch (error) {
            this.log(`\nError: ${error.response.data.message}\n`);
            return;
        }
        const validFiles = helper_1.Helper.getValidFiles(basePath).map((validFile) => {
            let name = basePath === '.' ? validFile.path : validFile.path.replace(basePath, '');
            if (name.startsWith('/')) {
                name = name.slice(1);
            }
            return { name, sha: validFile.sha };
        });
        const newFiles = [];
        const modifiedFiles = [];
        const unmodifiedFiles = [];
        const deletedFiles = [];
        let reportDto = null;
        try {
            const reportSlug = helper_1.Helper.slug(kysoConfigFile.title);
            const resultReport = await api.getReportByTeamIdAndSlug(teamId, reportSlug);
            reportDto = resultReport.data;
            const resultFiles = await api.getReportFiles(reportDto.id, reportDto.last_version);
            const reportFiles = resultFiles.data;
            validFiles.forEach((validFile) => {
                const indexFile = reportFiles.findIndex((reportFile) => reportFile.name === validFile.name);
                if (indexFile === -1) {
                    newFiles.push(validFile.name);
                }
                else if (reportFiles[indexFile].sha === validFile.sha) {
                    unmodifiedFiles.push(reportFiles[indexFile].name);
                }
                else {
                    modifiedFiles.push(reportFiles[indexFile].name);
                }
            });
            reportFiles.forEach((reportFile) => {
                const indexFile = validFiles.findIndex((validFile) => reportFile.sha === validFile.sha && reportFile.name === validFile.name);
                if (indexFile === -1) {
                    const indexModified = modifiedFiles.findIndex((modifiedFile) => modifiedFile === reportFile.name);
                    if (indexModified === -1) {
                        deletedFiles.push(reportFile.name);
                    }
                }
            });
        }
        catch (error) {
            const errorResponse = error.response.data;
            if (errorResponse.statusCode === 404) {
                this.error(`Report '${kysoConfigFile.title}' not found in '${kysoConfigFile.team}' channel of '${kysoConfigFile.organization}' organization`);
            }
        }
        this.log(`\nReport '${reportDto.title}'\n`);
        this.log(`Url: ${kysoCredentials.kysoInstallUrl}/${reportDto.organization_sluglified_name}/${reportDto.team_sluglified_name}/${reportDto.name}`);
        this.log(`Organization: ${kysoConfigFile.organization}`);
        this.log(`Channel: ${kysoConfigFile.team}`);
        this.log(`Version: ${reportDto.last_version}`);
        this.log(`Created at: ${(0, moment_1.default)(reportDto.created_at).format('DD/MM/YYYY HH:mm:ss')}`);
        this.log(`Updated at: ${(0, moment_1.default)(reportDto.updated_at).format('DD/MM/YYYY HH:mm:ss')}`);
        if (unmodifiedFiles.length === 0 && newFiles.length === 0 && modifiedFiles.length === 0 && deletedFiles.length === 0) {
            this.log(`\nReport is up to date`);
        }
        else {
            if (unmodifiedFiles.length > 0) {
                this.log(`\nUnmodified files: ${unmodifiedFiles.length}`);
            }
            if (newFiles.length > 0) {
                this.log(`\nNew files: ${newFiles.length}`);
                newFiles.forEach((newFile) => this.log(color_1.default.green(`\t${newFile}`)));
            }
            if (modifiedFiles.length > 0) {
                this.log(`\nModified files: ${modifiedFiles.length}`);
                modifiedFiles.forEach((modifiedFile) => this.log(color_1.default.yellow(`\t${modifiedFile}`)));
            }
            if (deletedFiles.length > 0) {
                this.log(`\nDeleted files: ${deletedFiles.length}`);
                deletedFiles.forEach((deletedFile) => this.log(color_1.default.red(`\t${deletedFile}`)));
            }
        }
        this.log(``);
    }
    async evaluatePath(basePath) {
        // Check if report is a multiple report
        const files = (0, fs_1.readdirSync)(basePath).map((file) => (0, path_1.join)(basePath, file));
        let mainKysoConfigFile = null;
        const data = helper_1.Helper.findKysoConfigFile(files);
        if (!data.valid) {
            this.log(`Error in Kyso config file: ${data.message}`);
            return;
        }
        mainKysoConfigFile = data.kysoConfigFile;
        if (mainKysoConfigFile === null || mainKysoConfigFile === void 0 ? void 0 : mainKysoConfigFile.reports) {
            this.log(`\n${mainKysoConfigFile.reports.length} ${mainKysoConfigFile.reports.length > 1 ? 'reports' : 'report'} found\n`);
            for (const reportFolder of mainKysoConfigFile.reports) {
                // Check if folder exists
                const reportPath = (0, path_1.join)(basePath, reportFolder);
                if (!(0, fs_1.existsSync)(reportPath)) {
                    this.error(`Report '${reportFolder}' folder does not exist.`);
                }
                const filesBasePath = (0, fs_1.readdirSync)(reportPath).map((file) => (0, path_1.join)(reportPath, file));
                const { valid, message } = helper_1.Helper.findKysoConfigFile(filesBasePath);
                if (!valid) {
                    this.error(`Folder '${reportFolder}' does not have a valid Kyso config file: ${message}`);
                }
                await this.reportStatus(reportFolder, reportPath);
            }
        }
        else {
            const parts = basePath.split('/');
            const reportFolder = parts[parts.length - 1];
            await this.reportStatus(reportFolder, basePath);
        }
    }
    async run() {
        const { flags } = await this.parse(Status);
        await (0, interactive_login_1.launchInteractiveLoginIfNotLogged)();
        if (!(0, fs_1.existsSync)(flags.path)) {
            this.error('Invalid path');
        }
        if (!(0, fs_1.lstatSync)(flags.path).isDirectory()) {
            this.error('Path must be a directory');
        }
        const basePath = (0, path_1.resolve)(flags.path);
        await this.evaluatePath(basePath);
    }
}
exports.default = Status;
Status.description = 'Show the working tree status';
Status.examples = [`$ kyso status --path ./my-report`];
Status.flags = {
    path: core_1.Flags.string({
        char: 'p',
        description: 'Path to root folder of the report to push. Default is "."',
        required: false,
        default: '.',
    }),
};
Status.args = [];
