"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const kyso_model_1 = require("@kyso-io/kyso-model");
const kyso_store_1 = require("@kyso-io/kyso-store");
const jwt_decode_1 = tslib_1.__importDefault(require("jwt-decode"));
const inquirer = require("inquirer");
const core_1 = require("@oclif/core");
const crypto = tslib_1.__importStar(require("crypto"));
const interactive_login_1 = require("../../helpers/interactive-login");
const kyso_command_1 = require("../kyso-command");
const helper_1 = require("../../helpers/helper");
class AddUsers extends kyso_command_1.KysoCommand {
    async createUser(api, email, yes, silent) {
        const data = email.split(':');
        let displayName = '';
        let extractedEmail = '';
        let username = '';
        if (data.length === 2) {
            displayName = data[0];
            extractedEmail = data[1];
        }
        else {
            extractedEmail = data[0];
        }
        username = helper_1.Helper.isEmail(extractedEmail) ? extractedEmail.split('@')[0] : extractedEmail;
        displayName = displayName || username;
        const wishlist = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz';
        const length = 30;
        const randomPassword = Array.from(crypto.randomFillSync(new Uint32Array(length)))
            .map((x) => wishlist[x % wishlist.length])
            .join('');
        const signUpDto = new kyso_model_1.SignUpDto(extractedEmail, username, displayName, randomPassword, silent);
        if (!yes) {
            if (!helper_1.Helper.isEmail(signUpDto.email)) {
                const emailResponse = await inquirer.prompt([
                    {
                        type: 'input',
                        name: 'email',
                        message: 'What is the email of the user?',
                        validate(email) {
                            if (email === '') {
                                return 'Email cannot be empty';
                            }
                            if (!helper_1.Helper.isEmail(email)) {
                                return 'Email is not valid';
                            }
                            return true;
                        },
                    },
                ]);
                signUpDto.email = emailResponse.email;
            }
            const usernameResponse = await inquirer.prompt([
                {
                    type: 'input',
                    name: 'username',
                    message: 'What is the username of the user?',
                    validate(username) {
                        if (username === '') {
                            return 'Username cannot be empty';
                        }
                        return true;
                    },
                },
            ]);
            signUpDto.username = usernameResponse.username;
            const displayNameResponse = await inquirer.prompt([
                {
                    type: 'input',
                    name: 'display_name',
                    message: 'What is the name of the user?',
                    validate(display_name) {
                        if (display_name === '') {
                            return 'Name cannot be empty';
                        }
                        return true;
                    },
                },
            ]);
            signUpDto.display_name = displayNameResponse.display_name;
            const passwordResponse = await inquirer.prompt([
                {
                    type: 'password',
                    name: 'password',
                    message: 'What is the password of the user?',
                    validate(password) {
                        if (password === '') {
                            return 'Password cannot be empty';
                        }
                        return true;
                    },
                },
            ]);
            signUpDto.password = passwordResponse.password;
            await inquirer.prompt([
                {
                    type: 'password',
                    name: 'password_confirmation',
                    message: 'What is the password confirmation of the user?',
                    validate(password_confirmation) {
                        if (password_confirmation === '') {
                            return 'Password confirmation cannot be empty';
                        }
                        if (password_confirmation !== passwordResponse.password) {
                            return 'Password confirmation does not match';
                        }
                        return true;
                    },
                },
            ]);
        }
        try {
            const userResult = await api.signup(signUpDto);
            const user = userResult.data;
            const kysoCredentials = kyso_command_1.KysoCommand.getCredentials();
            this.log(`\nUser '${user.email} - ${user.display_name}' created. Visit its page ${kysoCredentials.kysoInstallUrl}/user/${user.username}\n`);
        }
        catch (e) {
            const errorResponse = e.response.data;
            this.log(`Error creating the user '${signUpDto.email} - ${signUpDto.display_name}': ${errorResponse.message}`);
        }
    }
    async run() {
        const { args, flags } = await this.parse(AddUsers);
        const emails = args.list_of_emails.split(',');
        await (0, interactive_login_1.launchInteractiveLoginIfNotLogged)();
        const kysoCredentials = kyso_command_1.KysoCommand.getCredentials();
        const decoded = (0, jwt_decode_1.default)(kysoCredentials.token);
        const api = new kyso_store_1.Api();
        api.configure(`${kysoCredentials.kysoInstallUrl}/api/v1`, kysoCredentials === null || kysoCredentials === void 0 ? void 0 : kysoCredentials.token);
        let tokenPermissions = null;
        try {
            const resultPermissions = await api.getUserPermissions(decoded.payload.username);
            tokenPermissions = resultPermissions.data;
        }
        catch (e) {
            this.error('Error getting user permissions');
        }
        if (!tokenPermissions.global || !helper_1.Helper.isGlobalAdmin(tokenPermissions)) {
            this.error("You don't have permissions to add users");
        }
        for (const email of emails) {
            await this.createUser(api, email, flags.yes, flags.silent);
        }
    }
}
exports.default = AddUsers;
AddUsers.description = 'Add users to the system';
AddUsers.examples = [
    `How to use: \n\t$ kyso user add <display_name>:<email@email.io>,<email2@email.io>`,
    `Prompting: \n\t$ kyso user add Emilio:emilio@kyso.io,amidala@kyso.io`,
    `No prompting: \n\t$ kyso user add Emilio:emilio@kyso.io,amidala@kyso.io --yes`,
    `Display names with empty spaces: \n\t$ kyso user add "Emilio Pastor:emilio@kyso.io,amidala@kyso.io" --yes`,
];
AddUsers.flags = {
    yes: core_1.Flags.boolean({
        char: 'y',
        description: "Don't ask with prompter and use default values. The password will be autogenerated but not shown. The user will use the recover password procedure",
        required: false,
        default: false,
    }),
    silent: core_1.Flags.boolean({
        char: 's',
        description: 'Create users without sending notifications',
        required: false,
        default: false,
    }),
};
AddUsers.args = [
    {
        name: 'list_of_emails',
        description: 'List of emails separated by commas email1@kyso.io,email2@kyso.io. Display name can be specified as well separated by : --> Email1Name:email1@kyso.io,Email2Name:email2@kyso.io',
        required: true,
    },
];
