"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const kyso_store_1 = require("@kyso-io/kyso-store");
const jwt_decode_1 = tslib_1.__importDefault(require("jwt-decode"));
const inquirer = require("inquirer");
const interactive_login_1 = require("../../helpers/interactive-login");
const kyso_command_1 = require("../kyso-command");
const helper_1 = require("../../helpers/helper");
class DeleteUsers extends kyso_command_1.KysoCommand {
    async deleteUser(api, email) {
        let desiredEmail;
        if (!helper_1.Helper.isEmail(email)) {
            const emailResponse = await inquirer.prompt([
                {
                    type: 'input',
                    name: 'email',
                    message: 'What is the email of the user?',
                    validate(email) {
                        if (email === '') {
                            return 'Email cannot be empty';
                        }
                        if (!helper_1.Helper.isEmail(email)) {
                            return 'Email is not valid';
                        }
                        return true;
                    },
                },
            ]);
            desiredEmail = emailResponse.email;
        }
        else {
            desiredEmail = email;
        }
        let userDto = null;
        try {
            const usersResponse = await api.getUsers({ userIds: [], page: 1, per_page: 1000, sort: 'email', search: encodeURIComponent(desiredEmail) });
            const users = usersResponse.data;
            if (users.length === 0) {
                this.log(`Error: User with email '${desiredEmail}' not found`);
                return;
            }
            const index = users.findIndex((user) => user.email === desiredEmail);
            if (index === -1) {
                this.log(`Error: User with email '${desiredEmail}' not found`);
                return;
            }
            userDto = users[index];
        }
        catch (e) {
            const errorResponse = e.response.data;
            this.log(`Error getting the user '${desiredEmail}': ${errorResponse.message}`);
        }
        try {
            await api.deleteUser(userDto.id);
            this.log(`User '${desiredEmail}' deleted`);
        }
        catch (e) {
            const errorResponse = e.response.data;
            this.log(`Error deleting the user '${desiredEmail}': ${errorResponse.message}`);
        }
    }
    async run() {
        const { args } = await this.parse(DeleteUsers);
        const emails = args.list_of_emails.split(',');
        await (0, interactive_login_1.launchInteractiveLoginIfNotLogged)();
        const kysoCredentials = kyso_command_1.KysoCommand.getCredentials();
        const decoded = (0, jwt_decode_1.default)(kysoCredentials.token);
        const api = new kyso_store_1.Api();
        api.configure(`${kysoCredentials.kysoInstallUrl}/api/v1`, kysoCredentials === null || kysoCredentials === void 0 ? void 0 : kysoCredentials.token);
        let tokenPermissions = null;
        try {
            const resultPermissions = await api.getUserPermissions(decoded.payload.username);
            tokenPermissions = resultPermissions.data;
        }
        catch (e) {
            this.error('Error getting user permissions');
        }
        if (!tokenPermissions.global || !helper_1.Helper.isGlobalAdmin(tokenPermissions)) {
            this.error("You don't have permissions to delete users");
        }
        for (const email of emails) {
            await this.deleteUser(api, email);
        }
    }
}
exports.default = DeleteUsers;
DeleteUsers.description = 'Delete users from the system';
DeleteUsers.examples = [`$ kyso user delete <list_of_emails>`];
DeleteUsers.args = [
    {
        name: 'list_of_emails',
        description: 'List of emails separated by commas',
        required: true,
    },
];
