"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const kyso_store_1 = require("@kyso-io/kyso-store");
const core_1 = require("@oclif/core");
const axios_1 = tslib_1.__importDefault(require("axios"));
const fs_1 = require("fs");
const jwt_decode_1 = tslib_1.__importDefault(require("jwt-decode"));
const path_1 = require("path");
const helper_1 = require("../../../helpers/helper");
const interactive_login_1 = require("../../../helpers/interactive-login");
const kyso_command_1 = require("../../kyso-command");
class DownloadUserPhoto extends kyso_command_1.KysoCommand {
    async run() {
        const { args, flags } = await this.parse(DownloadUserPhoto);
        await (0, interactive_login_1.launchInteractiveLoginIfNotLogged)();
        const kysoCredentials = kyso_command_1.KysoCommand.getCredentials();
        const api = new kyso_store_1.Api();
        api.configure(`${kysoCredentials.kysoInstallUrl}/api/v1`, kysoCredentials === null || kysoCredentials === void 0 ? void 0 : kysoCredentials.token);
        const decoded = (0, jwt_decode_1.default)(kysoCredentials.token);
        let tokenPermissions = null;
        try {
            const resultPermissions = await api.getUserPermissions(decoded.payload.username);
            tokenPermissions = resultPermissions.data;
        }
        catch (e) {
            this.error('Error getting user permissions');
        }
        if (!helper_1.Helper.isGlobalAdmin(tokenPermissions)) {
            this.error("You don't have permissions to delete users");
        }
        let userDto = null;
        try {
            const usersResponse = await api.getUsers({ userIds: [], page: 1, per_page: 1000, sort: 'email', search: encodeURIComponent(args.email) });
            const users = usersResponse.data;
            if (users.length === 0) {
                this.log(`Error: User with email '${args.email}' not found`);
                return;
            }
            const index = users.findIndex((user) => user.email === args.email);
            if (index === -1) {
                this.log(`Error: User with email '${args.email}' not found`);
                return;
            }
            userDto = users[index];
        }
        catch (e) {
            const errorResponse = e.response.data;
            this.log(`Error getting the user '${args.email}': ${errorResponse.message}`);
        }
        if (!userDto.avatar_url) {
            this.error('User has no photo');
        }
        try {
            const imageUrl = userDto.avatar_url.startsWith('http') ? userDto.avatar_url : kysoCredentials.kysoInstallUrl + userDto.avatar_url;
            const axiosResponse = await axios_1.default.get(imageUrl, { responseType: 'arraybuffer' });
            const urlParts = axiosResponse.request.res.responseUrl.split('/');
            if (urlParts.length === 0) {
                this.error('Error downloading photo');
            }
            const fileName = urlParts[urlParts.length - 1];
            if (!fileName.includes('.')) {
                this.error('User profile photo has no extension');
            }
            const filePath = (0, path_1.resolve)((0, path_1.join)(flags.path, fileName));
            (0, fs_1.writeFileSync)(filePath, axiosResponse.data, 'binary');
            this.log(`Photo downloaded to ${filePath}`);
        }
        catch (e) {
            this.error('Error downloading photo');
        }
    }
}
exports.default = DownloadUserPhoto;
DownloadUserPhoto.description = 'Download user photo from Kyso';
DownloadUserPhoto.examples = [`$ kyso user download photo <email>`, `$ kyso profile download photo <email> -p <path>`];
DownloadUserPhoto.flags = {
    path: core_1.Flags.string({
        char: 'p',
        description: 'Destination folder in which the photo will be downloaded',
        required: false,
        default: '.',
    }),
};
DownloadUserPhoto.args = [
    {
        name: 'email',
        required: true,
        description: 'Email of the user',
    },
];
