"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const kyso_store_1 = require("@kyso-io/kyso-store");
const core_1 = require("@oclif/core");
const axios_1 = tslib_1.__importDefault(require("axios"));
const fs_1 = require("fs");
const jsYaml = tslib_1.__importStar(require("js-yaml"));
const jwt_decode_1 = tslib_1.__importDefault(require("jwt-decode"));
const path_1 = require("path");
const helper_1 = require("../../helpers/helper");
const interactive_login_1 = require("../../helpers/interactive-login");
const kyso_command_1 = require("../kyso-command");
class GetUsers extends kyso_command_1.KysoCommand {
    async getProfileData(api, email, getImages) {
        let userDto = null;
        let profileData = null;
        try {
            const usersResponse = await api.getUsers({ userIds: [], page: 1, per_page: 1000, sort: 'email', search: encodeURIComponent(email) });
            const users = usersResponse.data;
            if (users.length === 0) {
                this.log(`Error: User with email '${email}' not found`);
                return;
            }
            const index = users.findIndex((user) => user.email === email);
            if (index === -1) {
                this.log(`Error: User with email '${email}' not found`);
                return;
            }
            userDto = users[index];
            profileData = {
                email: userDto.email,
                username: userDto.username,
                name: userDto.name,
                display_name: userDto.display_name,
                bio: userDto.bio,
                location: userDto.location,
                link: userDto.link,
            };
        }
        catch (e) {
            const errorResponse = e.response.data;
            this.log(`Error getting the user '${email}': ${errorResponse.message}`);
            return profileData;
        }
        if (getImages) {
            const kysoCredentials = kyso_command_1.KysoCommand.getCredentials();
            if (userDto.avatar_url) {
                try {
                    const imageUrl = userDto.avatar_url.startsWith('http') ? userDto.avatar_url : kysoCredentials.kysoInstallUrl + userDto.avatar_url;
                    const axiosResponse = await axios_1.default.get(imageUrl, { responseType: 'text', responseEncoding: 'base64' });
                    if (axiosResponse.status === 200) {
                        profileData.photo = `data:${axiosResponse.headers['content-type']};base64,${axiosResponse.data}`;
                    }
                }
                catch (e) {
                    console.log(e);
                }
            }
            if (userDto.background_image_url) {
                try {
                    const backgroundImageUrl = userDto.background_image_url.startsWith('http') ? userDto.background_image_url : kysoCredentials.kysoInstallUrl + userDto.background_image_url;
                    const axiosResponse = await axios_1.default.get(backgroundImageUrl, { responseType: 'text', responseEncoding: 'base64' });
                    if (axiosResponse.status === 200) {
                        profileData.background = `data:${axiosResponse.headers['content-type']};base64,${axiosResponse.data}`;
                    }
                }
                catch (e) {
                    console.log(e);
                }
            }
        }
        return profileData;
    }
    async run() {
        const { flags, args } = await this.parse(GetUsers);
        const emails = args.list_of_emails.split(',');
        await (0, interactive_login_1.launchInteractiveLoginIfNotLogged)();
        const kysoCredentials = kyso_command_1.KysoCommand.getCredentials();
        const decoded = (0, jwt_decode_1.default)(kysoCredentials.token);
        const api = new kyso_store_1.Api();
        api.configure(`${kysoCredentials.kysoInstallUrl}/api/v1`, kysoCredentials === null || kysoCredentials === void 0 ? void 0 : kysoCredentials.token);
        let tokenPermissions = null;
        try {
            const resultPermissions = await api.getUserPermissions(decoded.payload.username);
            tokenPermissions = resultPermissions.data;
        }
        catch (e) {
            this.error('Error getting user permissions');
        }
        if (!tokenPermissions.global || !helper_1.Helper.isGlobalAdmin(tokenPermissions)) {
            this.error("You don't have permissions to delete users");
        }
        const result = [];
        for (const email of emails) {
            const userData = await this.getProfileData(api, email, flags.images);
            if (userData) {
                result.push(userData);
            }
        }
        if (result.length === 0) {
            this.log('No users found');
            return;
        }
        const yamlData = jsYaml.dump(result);
        const yamlFilePath = (0, path_1.resolve)(args.yaml_file);
        (0, fs_1.writeFileSync)(yamlFilePath, yamlData);
        this.log(`Users data saved in ${yamlFilePath}`);
    }
}
exports.default = GetUsers;
GetUsers.description = 'Save in a yaml file users profile data';
GetUsers.examples = [`$ kyso user get <list_of_emails> <yaml_file>`, `$ kyso user get <list_of_emails> <yaml_file> --images`];
GetUsers.flags = {
    images: core_1.Flags.boolean({
        char: 'i',
        description: 'Get user image in base64 format',
        required: false,
        default: false,
    }),
};
GetUsers.args = [
    {
        name: 'list_of_emails',
        description: 'List of emails separated by commas',
        required: true,
    },
    {
        name: 'yaml_file',
        description: 'Yaml file where the users data will be saved',
        required: true,
    },
];
