"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const kyso_store_1 = require("@kyso-io/kyso-store");
const axios_1 = tslib_1.__importDefault(require("axios"));
const fs_1 = require("fs");
const jsYaml = tslib_1.__importStar(require("js-yaml"));
const jwt_decode_1 = tslib_1.__importDefault(require("jwt-decode"));
const path_1 = require("path");
const helper_1 = require("../../helpers/helper");
const interactive_login_1 = require("../../helpers/interactive-login");
const kyso_command_1 = require("../kyso-command");
class SetUsers extends kyso_command_1.KysoCommand {
    async updatePhoto(api, userId, photoBase64) {
        if (!photoBase64.startsWith('data:image/')) {
            this.error('Photo must be a base64 string');
        }
        try {
            // Create file from base64 string
            const formatImage = photoBase64.split(';')[0].split('/')[1];
            const imageFilePath = (0, path_1.join)(kyso_command_1.KysoCommand.DATA_DIRECTORY, `photo-${new Date().getTime()}.${formatImage}`);
            const buffer = Buffer.from(photoBase64.split(',')[1], 'base64');
            (0, fs_1.writeFileSync)(imageFilePath, buffer);
            const readStream = (0, fs_1.createReadStream)(imageFilePath);
            await api.uploadUserProfileImage(userId, readStream);
            (0, fs_1.unlinkSync)(imageFilePath);
        }
        catch (e) {
            this.log(`Error uploading profile photo: ${e.response.data.message}`);
        }
    }
    async updateBackgroundImage(api, userId, backgroundBase64) {
        if (!backgroundBase64.startsWith('data:image/')) {
            this.error('Background image must be a base64 string');
        }
        try {
            // Create file from base64 string
            const formatImage = backgroundBase64.split(';')[0].split('/')[1];
            const imageFilePath = (0, path_1.join)(kyso_command_1.KysoCommand.DATA_DIRECTORY, `background-${new Date().getTime()}.${formatImage}`);
            const buffer = Buffer.from(backgroundBase64.split(',')[1], 'base64');
            (0, fs_1.writeFileSync)(imageFilePath, buffer);
            const readStream = (0, fs_1.createReadStream)(imageFilePath);
            await api.uploadUserBackgroundImage(userId, readStream);
            (0, fs_1.unlinkSync)(imageFilePath);
        }
        catch (e) {
            this.log(`Error uploading background profile image: ${e.response.data.message}`);
        }
    }
    async updateUserData(api, profileData) {
        let userDto = null;
        let photoBase64 = null;
        let backgroundImageBase64 = null;
        try {
            const usersResponse = await api.getUsers({ userIds: [], page: 1, per_page: 1000, sort: 'email', search: encodeURIComponent(profileData.email) });
            const users = usersResponse.data;
            if (users.length === 0) {
                this.log(`Error: User with email '${profileData.email}' not found`);
                return;
            }
            const index = users.findIndex((user) => user.email === profileData.email);
            if (index === -1) {
                this.log(`Error: User with email '${profileData.email}' not found`);
                return;
            }
            userDto = users[index];
        }
        catch (e) {
            const errorResponse = e.response.data;
            this.log(`Error getting the user '${profileData.email}': ${errorResponse.message}`);
            return;
        }
        const kysoCredentials = kyso_command_1.KysoCommand.getCredentials();
        if (userDto.avatar_url) {
            try {
                const imageUrl = userDto.avatar_url.startsWith('http') ? userDto.avatar_url : kysoCredentials.kysoInstallUrl + userDto.avatar_url;
                const axiosResponse = await axios_1.default.get(imageUrl, { responseType: 'text', responseEncoding: 'base64' });
                if (axiosResponse.status === 200) {
                    photoBase64 = `data:${axiosResponse.headers['content-type']};base64,${axiosResponse.data}`;
                }
            }
            catch (e) {
                console.log(e);
            }
        }
        if (userDto.background_image_url) {
            try {
                const backgroundImageUrl = userDto.background_image_url.startsWith('http') ? userDto.background_image_url : kysoCredentials.kysoInstallUrl + userDto.background_image_url;
                const axiosResponse = await axios_1.default.get(backgroundImageUrl, { responseType: 'text', responseEncoding: 'base64' });
                if (axiosResponse.status === 200) {
                    backgroundImageBase64 = `data:${axiosResponse.headers['content-type']};base64,${axiosResponse.data}`;
                }
            }
            catch (e) {
                console.log(e);
            }
        }
        let updatedPhoto = false;
        if ((!photoBase64 && profileData.photo) || (photoBase64 && profileData.photo && photoBase64 !== profileData.photo)) {
            await this.updatePhoto(api, userDto.id, profileData.photo);
            updatedPhoto = true;
        }
        let updatedBackgroundImage = false;
        if ((!backgroundImageBase64 && profileData.background) || (backgroundImageBase64 && profileData.background && backgroundImageBase64 !== profileData.background)) {
            await this.updateBackgroundImage(api, userDto.id, profileData.background);
            updatedBackgroundImage = true;
        }
        const updateUserRequestDto = {};
        if (profileData.name && profileData.name !== userDto.name) {
            updateUserRequestDto.name = profileData.name;
        }
        if (profileData.display_name && profileData.display_name !== userDto.display_name) {
            updateUserRequestDto.display_name = profileData.display_name;
        }
        if (profileData.location && profileData.location !== userDto.location) {
            updateUserRequestDto.location = profileData.location;
        }
        if (profileData.link && profileData.link !== userDto.link) {
            updateUserRequestDto.link = profileData.link;
        }
        if (profileData.bio && profileData.bio !== userDto.bio) {
            updateUserRequestDto.bio = profileData.bio;
        }
        if (Object.keys(updateUserRequestDto).length > 0) {
            try {
                await api.updateUser(userDto.id, updateUserRequestDto);
                this.log(`User '${profileData.email}' updated`);
            }
            catch (e) {
                this.error(`Error updating user ${profileData.email}: ${e.response.data.message}`);
            }
        }
        else if (!updatedPhoto && !updatedBackgroundImage) {
            this.log(`No changes for the user '${profileData.email}'`);
        }
        else {
            this.log(`User '${profileData.email}' updated`);
        }
    }
    async run() {
        const { args } = await this.parse(SetUsers);
        if (!args.yaml_file.endsWith('.yaml') && !args.yaml_file.endsWith('.yml')) {
            this.error('File is not a yaml file');
        }
        if (!(0, fs_1.existsSync)(args.yaml_file)) {
            this.log(`File ${args.yaml_file} does not exist`);
            return;
        }
        const yamlFileContent = (0, fs_1.readFileSync)(args.yaml_file, 'utf8');
        let profilesData = null;
        try {
            profilesData = jsYaml.load(yamlFileContent);
            if (!Array.isArray(profilesData)) {
                this.error('File is not an array of users profile');
            }
        }
        catch (e) {
            this.error('File is not a valid yaml file');
        }
        await (0, interactive_login_1.launchInteractiveLoginIfNotLogged)();
        const kysoCredentials = kyso_command_1.KysoCommand.getCredentials();
        const decoded = (0, jwt_decode_1.default)(kysoCredentials.token);
        const api = new kyso_store_1.Api();
        api.configure(`${kysoCredentials.kysoInstallUrl}/api/v1`, kysoCredentials === null || kysoCredentials === void 0 ? void 0 : kysoCredentials.token);
        let tokenPermissions = null;
        try {
            const resultPermissions = await api.getUserPermissions(decoded.payload.username);
            tokenPermissions = resultPermissions.data;
        }
        catch (e) {
            this.error('Error getting user permissions');
        }
        if (!tokenPermissions.global || !helper_1.Helper.isGlobalAdmin(tokenPermissions)) {
            this.error("You don't have permissions to delete users");
        }
        let index = 0;
        for (const profileData of profilesData) {
            if (!helper_1.Helper.isEmail(profileData.email)) {
                this.log(`Email of item in position ${index}' is not a valid email`);
                index++;
                continue;
            }
            await this.updateUserData(api, profileData);
            index++;
        }
    }
}
exports.default = SetUsers;
SetUsers.description = 'Update users profile data given yaml file';
SetUsers.examples = [`$ kyso user set <yaml_file>`];
SetUsers.args = [
    {
        name: 'yaml_file',
        description: 'Yaml file with users profile data',
        required: true,
    },
];
