"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const kyso_store_1 = require("@kyso-io/kyso-store");
const fs_1 = require("fs");
const jwt_decode_1 = tslib_1.__importDefault(require("jwt-decode"));
const helper_1 = require("../../../helpers/helper");
const interactive_login_1 = require("../../../helpers/interactive-login");
const kyso_command_1 = require("../../kyso-command");
class UploadUserBackground extends kyso_command_1.KysoCommand {
    async run() {
        const { args } = await this.parse(UploadUserBackground);
        await (0, interactive_login_1.launchInteractiveLoginIfNotLogged)();
        const kysoCredentials = kyso_command_1.KysoCommand.getCredentials();
        const api = new kyso_store_1.Api();
        api.configure(`${kysoCredentials.kysoInstallUrl}/api/v1`, kysoCredentials === null || kysoCredentials === void 0 ? void 0 : kysoCredentials.token);
        const decoded = (0, jwt_decode_1.default)(kysoCredentials.token);
        let tokenPermissions = null;
        try {
            const resultPermissions = await api.getUserPermissions(decoded.payload.username);
            tokenPermissions = resultPermissions.data;
        }
        catch (e) {
            this.error('Error getting user permissions');
        }
        if (!helper_1.Helper.isGlobalAdmin(tokenPermissions)) {
            this.error("You don't have permissions to delete users");
        }
        let userDto = null;
        try {
            const usersResponse = await api.getUsers({ userIds: [], page: 1, per_page: 1000, sort: 'email', search: encodeURIComponent(args.email) });
            const users = usersResponse.data;
            if (users.length === 0) {
                this.log(`Error: User with email '${args.email}' not found`);
                return;
            }
            const index = users.findIndex((user) => user.email === args.email);
            if (index === -1) {
                this.log(`Error: User with email '${args.email}' not found`);
                return;
            }
            userDto = users[index];
        }
        catch (e) {
            const errorResponse = e.response.data;
            this.log(`Error getting the user '${args.email}': ${errorResponse.message}`);
        }
        if (!userDto.avatar_url) {
            this.error('User has no background');
        }
        if (!(0, fs_1.existsSync)(args.path)) {
            this.log(`File ${args.path} does not exist`);
            return;
        }
        if (!helper_1.Helper.isImage(args.path)) {
            this.log(`File ${args.path} is not an image. Valid formats are: png, jpg, jpeg, gif`);
            return;
        }
        await (0, interactive_login_1.launchInteractiveLoginIfNotLogged)();
        try {
            const readStream = (0, fs_1.createReadStream)(args.path);
            await api.uploadUserBackgroundImage(userDto.id, readStream);
            this.log(`Background image uploaded successfully`);
        }
        catch (e) {
            this.log(`Error uploading background image: ${e.response.data.message}`);
        }
    }
}
exports.default = UploadUserBackground;
UploadUserBackground.description = 'Upload user background to Kyso';
UploadUserBackground.examples = [`$ kyso user upload background <email> <path>`];
UploadUserBackground.args = [
    {
        name: 'email',
        required: true,
        description: 'Email of the user',
    },
    {
        name: 'path',
        required: true,
        description: 'Path to the image',
    },
];
