"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Helper = void 0;
const tslib_1 = require("tslib");
const kyso_model_1 = require("@kyso-io/kyso-model");
const kyso_store_1 = require("@kyso-io/kyso-store");
const fs_1 = require("fs");
const ignore_1 = tslib_1.__importDefault(require("ignore"));
const inquirer_1 = tslib_1.__importDefault(require("inquirer"));
const path_1 = tslib_1.__importStar(require("path"));
const sha256_file_1 = tslib_1.__importDefault(require("sha256-file"));
const slugify_1 = tslib_1.__importDefault(require("slugify"));
const kyso_command_1 = require("../commands/kyso-command");
class Helper {
    static async getOrganizationFromSlugSecurely(organizationParameter, credentials) {
        // Slug the organization to ensure that if someone introduced the name of the organization in
        // capital letters we are going to be able to answer properly
        const slugifiedOrganization = Helper.slug(organizationParameter);
        let result;
        try {
            const api = new kyso_store_1.Api();
            api.configure(`${credentials.kysoInstallUrl}/api/v1`, credentials === null || credentials === void 0 ? void 0 : credentials.token);
            result = await api.getOrganizationBySlug(slugifiedOrganization);
        }
        catch (e) {
            console.log(`Can't retrieve organization ${organizationParameter}`);
            if (organizationParameter !== slugifiedOrganization) {
                console.log(`Detected a non-slug value, automatically slugified to ${slugifiedOrganization}`);
            }
            console.log('Please check that the name (or slug) of the organization is correct');
            throw new Error('Please check that the name (or slug) of the organization is correct');
        }
        return result;
    }
    static async getChannelFromSlugSecurely(organization, channelParameter, credentials, silent) {
        // Slug the organization to ensure that if someone introduced the name of the organization in
        // capital letters we are going to be able to answer properly
        const slugifiedChannel = Helper.slug(channelParameter);
        let result;
        try {
            const api = new kyso_store_1.Api();
            api.configure(`${credentials.kysoInstallUrl}/api/v1`, credentials === null || credentials === void 0 ? void 0 : credentials.token, organization.sluglified_name);
            result = await api.getTeamBySlug(organization.id, slugifiedChannel);
        }
        catch (e) {
            if (!silent) {
                console.log(`Can't retrieve channel ${channelParameter} from organization ${organization.display_name}`);
                if (channelParameter !== slugifiedChannel) {
                    console.log(`Detected a non-slug value, automatically slugified to ${slugifiedChannel}`);
                }
                console.log('Please check that the name (or slug) of the channel is correct');
            }
            throw new Error('Please check that the name (or slug) of the channel is correct');
        }
        return result;
    }
    /**
     * Receives an array of organizations (name or slugs, or intercalated)
     *
     * Checks that every organization exists and extracts its slug name
     *
     * Returns an array of checked slug names. If some of these organizations are not valid
     * or don't exists, throws an exception
     *
     * @param list_of_orgs List of orgs (names or slugs) separated by commas
     * @returns Ensured list of slugified names.
     */
    static async getRealOrganizationSlugFromStringArray(list_of_orgs, credentials) {
        // Slug the organization to ensure that if someone introduced the name of the organization in
        // capital letters we are going to be able to answer properly
        const organizationList = list_of_orgs.split(',');
        const organizationSlugs = [];
        for (const org of organizationList) {
            const secureOrg = await Helper.getOrganizationFromSlugSecurely(org, credentials);
            organizationSlugs.push(secureOrg.data.sluglified_name);
        }
        return organizationSlugs;
    }
    /**
     * Receives an array of channels (name or slugs, or intercalated)
     *
     * Checks that every channel exists and extracts its slug name
     *
     * Returns an array of checked slug names. If some of these channels are not valid
     * or don't exists, throws an exception
     *
     * @param organization Organization in which channels must be subscribed
     * @param list_of_channels List of channels (names or slugs) separated by commas
     * @returns Ensured list of slugified names.
     */
    static async getRealChannelsSlugFromStringArray(organization, list_of_channels, credentials) {
        // Slug the organization to ensure that if someone introduced the name of the organization in
        // capital letters we are going to be able to answer properly
        const channelsList = list_of_channels.split(',');
        const channelsSlug = [];
        for (const channel of channelsList) {
            const secureChannel = await Helper.getChannelFromSlugSecurely(organization, channel, credentials);
            channelsSlug.push(secureChannel.data.sluglified_name);
        }
        return channelsSlug;
    }
    static findKysoConfigFile(files, basePath) {
        let data = {
            valid: false,
            message: null,
            kysoConfigFile: null,
        };
        const allKysoYamlFiles = files.filter((x) => x.endsWith('kyso.yaml') || x.endsWith('kyso.json') || x.endsWith('kyso.yml'));
        const sortedKysoYamlFiles = allKysoYamlFiles.sort((x, y) => x.length - y.length);
        const kysoConfigPath = sortedKysoYamlFiles && sortedKysoYamlFiles.length > 0 ? sortedKysoYamlFiles[0] : null;
        if (!kysoConfigPath) {
            return { kysoConfigFile: null, kysoConfigPath: null, valid: false, message: 'No kyso config file found' };
        }
        if (kysoConfigPath.endsWith('yml') || kysoConfigPath.endsWith('yaml')) {
            try {
                data = kyso_model_1.KysoConfigFile.fromYaml((0, fs_1.readFileSync)(kysoConfigPath, 'utf8'));
            }
            catch (error) {
                throw new Error(`Error parsing ${kysoConfigPath}: ${error.message}`);
            }
        }
        else if (kysoConfigPath.endsWith('json')) {
            try {
                data = kyso_model_1.KysoConfigFile.fromJSON((0, fs_1.readFileSync)(kysoConfigPath, 'utf8').toString());
            }
            catch (error) {
                throw new Error(`Error parsing ${kysoConfigPath}: ${error.message}`);
            }
        }
        else {
            data.message = 'No kyso config file found.';
        }
        // To allow the possibility to rename team to channel
        if ((data === null || data === void 0 ? void 0 : data.kysoConfigFile) && data.kysoConfigFile.channel && !data.kysoConfigFile.team) {
            data.kysoConfigFile.team = data.kysoConfigFile.channel;
        }
        return { kysoConfigFile: data.kysoConfigFile, kysoConfigPath, valid: data.valid, message: data.message };
    }
    static isEmail(email) {
        if (!email) {
            return false;
        }
        const emailRegex = new RegExp('^[a-zA-Z0-9.!#$%&’*+/=?^_`{|}~-]+@[a-zA-Z0-9-]+(?:.[a-zA-Z0-9-]+)*$');
        return emailRegex.test(email);
    }
    static isImage(name) {
        return name != null && (name.toLowerCase().endsWith('.png') || name.toLowerCase().endsWith('.jpg') || name.toLowerCase().endsWith('.jpeg') || name.toLowerCase().endsWith('.gif'));
    }
    static printErrorMessage(ex) {
        if (ex.hasOwnProperty('response')) {
            // It's an HTTP exception
            if (ex.response) {
                switch (ex.response.status) {
                    case 401: {
                        console.log("\nAuthorization failed. Please run 'kyso login' again\n");
                        break;
                    }
                    case 403: {
                        console.log("\n⛔ You don't have enough permissions to perform this action\n");
                        break;
                    }
                    case 400: {
                        console.log('\nBad request. Check the provided data.\n');
                        break;
                    }
                    default: {
                        console.log(`\n${ex.response.statusText}`);
                        break;
                    }
                }
            }
            else {
                // If response is null the object is different
                if (ex.message.includes('getaddrinfo ENOTFOUND')) {
                    console.log(`\n${kyso_command_1.KysoCommand.getCredentials().kysoInstallUrl} is not reachable. Please check your internet connection and ensure that your Kyso instance is available\n`);
                }
                else {
                    // We don't know the message, so just print it
                    console.log(`\n${ex.message}`);
                }
            }
        }
        else {
            // It's a common node exception
            if (ex.hasOwnProperty('message')) {
                if (ex.message.includes('ENOENT: no such file or directory')) {
                    console.log(`\nThe specified directory does not exist. Please create it before launching kyso\n`);
                    console.log(ex.message);
                }
                else {
                    // We don't know the message, so just print it
                    console.log(`\n${ex.message}`);
                }
            }
            else {
                console.log(`Unexpected error`);
            }
        }
    }
    static getAllFiles(dirPath, arrayOfFiles) {
        const files = (0, fs_1.readdirSync)(dirPath);
        arrayOfFiles = arrayOfFiles || [];
        for (const file of files) {
            if (file.startsWith('.') || file.endsWith('__MACOSX')) {
                continue;
            }
            if (!(0, fs_1.existsSync)(`${dirPath}/${file}`)) {
                continue;
            }
            if ((0, fs_1.statSync)(`${dirPath}/${file}`).isDirectory()) {
                arrayOfFiles = Helper.getAllFiles(`${dirPath}/${file}`, arrayOfFiles);
            }
            else {
                arrayOfFiles.push((0, path_1.join)(dirPath, '/', file));
            }
        }
        return arrayOfFiles;
    }
    static getValidFiles(dirPath) {
        let isSingleFile = false;
        if (!(0, fs_1.existsSync)(dirPath)) {
            throw new Error(`Path ${dirPath} not found`);
        }
        let files;
        if ((0, fs_1.lstatSync)(dirPath).isDirectory()) {
            files = (0, fs_1.readdirSync)(dirPath);
        }
        else if ((0, fs_1.lstatSync)(dirPath).isFile()) {
            files = [dirPath];
            isSingleFile = true;
        }
        const filesToIgnore = ['.git', '.DS_Store'];
        for (const file of files) {
            if (file === '.gitignore' || file === '.kysoignore') {
                // Read content of file
                const content = (0, fs_1.readFileSync)((0, path_1.join)(dirPath, file), 'utf8').toString();
                // Split content into lines
                const lines = content.split('\n');
                // Remove empty lines
                const filteredLines = lines.filter((line) => line.length > 0);
                // Add lines to ignored files if line is not in list
                for (const line of filteredLines) {
                    // Check duplicates
                    if (!filesToIgnore.includes(line)) {
                        filesToIgnore.push(line);
                    }
                }
            }
        }
        let filteredFiles = [];
        if (!isSingleFile) {
            // Remove ignored files defined in .gitignore or .kysoignore
            const ig = (0, ignore_1.default)().add(filesToIgnore);
            filteredFiles = files.filter((file) => !ig.ignores(file));
        }
        else {
            filteredFiles = files;
        }
        let validFiles = [];
        for (const file of filteredFiles) {
            let filePath = '';
            if (isSingleFile) {
                filePath = path_1.default.resolve(dirPath);
            }
            else {
                const sanitizedDirPath = dirPath.endsWith('/') ? dirPath : `${dirPath}/`;
                filePath = sanitizedDirPath + file;
            }
            const fileBaseName = path_1.default.basename(filePath);
            // For each file
            if (fileBaseName.startsWith('.')) {
                // Ignore hidden files
                continue;
            }
            // check if it is a directory
            if ((0, fs_1.lstatSync)(filePath).isDirectory()) {
                // Recursive call
                const folderFiles = Helper.getValidFiles(filePath);
                validFiles = [...validFiles, ...folderFiles];
            }
            else {
                validFiles.push({
                    path: filePath,
                    sha: (0, sha256_file_1.default)(filePath),
                });
            }
        }
        return validFiles;
    }
    static slug(url) {
        return (0, slugify_1.default)(url, {
            replacement: '-',
            lower: true,
            strict: true,
            trim: true,
        });
    }
    static isOrganizationAdmin(resourcePermissions) {
        if (!resourcePermissions) {
            return false;
        }
        if (!resourcePermissions.permissions || !Array.isArray(resourcePermissions.permissions)) {
            return false;
        }
        return resourcePermissions.permissions.includes(kyso_model_1.OrganizationPermissionsEnum.ADMIN);
    }
    static isGlobalAdmin(tokenPermissions) {
        if (!tokenPermissions) {
            return false;
        }
        if (!tokenPermissions.global || !Array.isArray(tokenPermissions.global)) {
            return false;
        }
        return tokenPermissions.global.includes(kyso_model_1.GlobalPermissionsEnum.GLOBAL_ADMIN);
    }
    static hasPermission(resourcePermissions, permission) {
        if (!resourcePermissions) {
            return false;
        }
        if (!resourcePermissions.permissions || !Array.isArray(resourcePermissions.permissions)) {
            return false;
        }
        return resourcePermissions.permissions.includes(permission);
    }
    static async askUserKysoInstallUrl(defaultKysoInstallUrl) {
        const kysoApiResponse = await inquirer_1.default.prompt([
            {
                name: 'kysoInstallUrl',
                message: 'What is the url of your kyso installation?',
                type: 'input',
                default: defaultKysoInstallUrl,
                validate(urlStr) {
                    let url;
                    try {
                        url = new URL(urlStr);
                    }
                    catch (_) {
                        return false;
                    }
                    return url.protocol === 'http:' || url.protocol === 'https:';
                },
                filter: (input) => {
                    if (input.endsWith('/')) {
                        input = input.slice(0, -1);
                    }
                    return input.trim();
                },
            },
        ]);
        const url = new URL(kysoApiResponse.kysoInstallUrl);
        return url.origin;
    }
    static parseFileSizeStr(fileSizeStr) {
        const units = ['b', 'kb', 'mb', 'gb', 'tb', 'pb', 'eb', 'zb', 'yb'];
        const size = parseFloat(fileSizeStr);
        const unit = fileSizeStr.replace(/[^a-z]/gi, '').toLowerCase();
        const power = units.indexOf(unit);
        return Math.floor(size * 1024 ** power);
    }
    /**
     * Sanitizes the incoming url parameter looking for auth credentials and
     * returns the URL without that credentials, if exists, and if not, returns
     * the same url.
     *
     * If any error happens, returns an empty string to avoid credentials leaking
     *
     * @param url
     * @returns
     */
    static sanitizeUrlBasicAuthentication(url) {
        try {
            const reg = /\/\/([^:]+):([^@]+)@/;
            const splittedUrl = url.split('?');
            const matches = splittedUrl[0].match(reg);
            if (matches && matches.length > 0) {
                return `${splittedUrl[0].replace(reg, '//')}${url.replace(splittedUrl[0], '')}`;
            }
            return url;
        }
        catch (e) {
            console.log('Error sanitizing url. Returning empty string to avoid credential leaking');
            return '';
        }
    }
}
exports.Helper = Helper;
